unit dihavBtn;

    {
     -------- TdihavButton -------
     -------- version 1.0 --------
     ---- By Mohammad Yousefi ----
     --- http://www.dihav.com/ ---
     - October 2010 - June 2012 --
     -- vahid_you2004@yahoo.com --
                                     }

interface

uses
  Classes, Windows, Messages, SysUtils, TypInfo, RTLConsts,
  {$if CompilerVersion > 22}
    Vcl.Controls, Vcl.Graphics, Vcl.Forms, Vcl.StdCtrls, Vcl.ActnList, Vcl.ImgList, Vcl.ExtCtrls
  {$else}
    Controls, Graphics, Forms, StdCtrls, ActnList, ImgList, ExtCtrls
  {$ifend}
  ;

type

  TdihavButton = class;
  TdihavButtonEffect = class;
  EScriptError = Exception;

  TdihavButtonStates = (bsActive, bsOver, bsDown, bsFocused, bsDisabled);
  TdihavButtonState = set of TdihavButtonStates;
  TdihavButtonEffectType = (etNormal, etActive, etOver, etDown, etFocused, etDisabled);

  TdihavButtonCustomMapping = procedure (Sender: TObject; AEffect: TdihavButtonEffect; U, V: Single; var X, Y: Single) of object;
  TdihavButtonCustomColoring = procedure (Sender: TObject; AEffect: TdihavButtonEffect; var R, G, B, A: Byte) of object;
  TdihavButtonCustomBackground = procedure (Sender: TObject; ACanvas: TCanvas; ARect: TRect) of object;
  TdihavButtonCustomAnimationScript = procedure (Sender: TObject; AEffect: TdihavButtonEffect; ALineNumber: Integer; AText: String) of object;
  TdihavButtonStateChanged = procedure (Sender: TObject; AOldState: TdihavButtonState) of object;

  TdihavButtonEffect = class(TPersistent)
  private
    FOwner: TdihavButton;
    FType: TdihavButtonEffectType;
    FPicture: TPicture;
    FImageIndex: TImageIndex;
    FImg, FImgMask: TBitmap;
    FColorShiftR, FColorShiftG, FColorShiftB, FColorShiftA: SmallInt;
    FColorRatioR, FColorRatioG, FColorRatioB, FColorRatioA: SmallInt;
    FColorShiftH, FColorShiftS, FColorShiftV: SmallInt;
    FColorRatioH, FColorRatioS, FColorRatioV: SmallInt;
    FInvertColors: Boolean;
    FGrayScale: Boolean;
    FMasked: Boolean;
    FMaskColor: TColor;
    FCustomColoring: Boolean;
    FWidthRatio, FHeightRatio: SmallInt;
    FRotationAngle: SmallInt;
    FCustomMapping: Boolean;
    FColorInversionR, FColorInversionG, FColorInversionB: Byte;
    FAnimateFromLastState: Boolean;
    FAnimationInterval, FFAnimationDuration: Cardinal;
    FAnimationScript: TStrings;
    FAnimatedEffect: Boolean;
    FReplayAnimation: Boolean;
    FAnimationScriptPos: Integer;
    FTimer: TTimer;
    FBrightness, FContrast: SmallInt;
    FShadow: Boolean;
    FShadowColor: TColor;
    FShadowX, FShadowY: Integer;
    FShadowAlpha: Byte;
    FShadowBlurRadius: Byte;
    FOverlayImage: Boolean;
    FMoveX, FMoveY: Integer;
    FApplyChangesNow: Boolean;
    FImgW, FImgH: Integer;

    procedure SetImageIndex(Value: TImageIndex);
    procedure SetPicture(Value: TPicture);
    procedure ImgChange(Sender: TObject);
    procedure SetColorShiftR(Value: SmallInt);
    procedure SetColorShiftG(Value: SmallInt);
    procedure SetColorShiftB(Value: SmallInt);
    procedure SetColorShiftA(Value: SmallInt);
    procedure SetColorRatioR(Value: SmallInt);
    procedure SetColorRatioG(Value: SmallInt);
    procedure SetColorRatioB(Value: SmallInt);
    procedure SetColorRatioA(Value: SmallInt);
    procedure SetColorShiftH(Value: SmallInt);
    procedure SetColorShiftS(Value: SmallInt);
    procedure SetColorShiftV(Value: SmallInt);
    procedure SetColorRatioH(Value: SmallInt);
    procedure SetColorRatioS(Value: SmallInt);
    procedure SetColorRatioV(Value: SmallInt);
    procedure SetInvertColors(Value: Boolean);
    procedure SetGrayScale(Value: Boolean);
    procedure SetMasked(Value: Boolean);
    procedure SetMaskColor(Value: TColor);
    procedure SetCustomColoring(Value: Boolean);
    procedure SetWidthRatio(Value: SmallInt);
    procedure SetHeightRatio(Value: SmallInt);
    procedure SetRotationAngle(Value: SmallInt);
    procedure SetCustomMapping(Value: Boolean);
    procedure SetColorInversionR(Value: Byte);
    procedure SetColorInversionG(Value: Byte);
    procedure SetColorInversionB(Value: Byte);
    procedure SetAnimationScript(Value: TStrings);
    procedure SetAnimatedEffect(Value: Boolean);
    procedure SetReplayAnimation(Value: Boolean);
    procedure TimerTick(Sender: TObject);
    procedure SetBrightness(Value: SmallInt);
    procedure SetContrast(Value: SmallInt);
    procedure SetShadow(Value: Boolean);
    procedure SetShadowColor(Value: TColor);
    procedure SetShadowX(Value: Integer);
    procedure SetShadowY(Value: Integer);
    procedure SetShadowAlpha(Value: Byte);
    procedure SetShadowBlurRadius(Value: Byte);
    procedure SetOverlayImage(Value: Boolean);
    function GetName: String;
    procedure SetMoveX(Value: Integer);
    procedure SetMoveY(Value: Integer);
    procedure PropertyChanged;
    procedure SetApplyChangesNow(Value: Boolean);
  protected
    FMaxSize: TSize;

    procedure GraphChanged;
    procedure CustomMap(var x, y: Single);
    procedure UpdateMaxSize;
    procedure Draw(Bitmap: TBitmap);
    procedure ContinueAnimation;
    procedure AssignTo(Dest: TPersistent); override;
    procedure SaveStateImages;
  public
    constructor Create(AOwner: TdihavButton; AType: TdihavButtonEffectType);
    destructor Destroy; override;
    procedure ResetAnimation;
    procedure Repaint;

    property Owner: TdihavButton read FOwner;
    property EffectType: TdihavButtonEffectType read FType;
    property ApplyChangesNow: Boolean read FApplyChangesNow write SetApplyChangesNow;
  published
    property Picture: TPicture read FPicture write SetPicture;
    property ImageIndex: TImageIndex read FImageIndex write SetImageIndex default -1;
    property ColorShiftR: SmallInt read FColorShiftR write SetColorShiftR default 0;
    property ColorShiftG: SmallInt read FColorShiftG write SetColorShiftG default 0;
    property ColorShiftB: SmallInt read FColorShiftB write SetColorShiftB default 0;
    property ColorShiftA: SmallInt read FColorShiftA write SetColorShiftA default 0;
    property ColorPercentageR: SmallInt read FColorRatioR write SetColorRatioR default 100;
    property ColorPercentageG: SmallInt read FColorRatioG write SetColorRatioG default 100;
    property ColorPercentageB: SmallInt read FColorRatioB write SetColorRatioB default 100;
    property ColorPercentageA: SmallInt read FColorRatioA write SetColorRatioA default 100;
    property ColorShiftH: SmallInt read FColorShiftH write SetColorShiftH default 0;
    property ColorShiftS: SmallInt read FColorShiftS write SetColorShiftS default 0;
    property ColorShiftV: SmallInt read FColorShiftV write SetColorShiftV default 0;
    property ColorPercentageH: SmallInt read FColorRatioH write SetColorRatioH default 100;
    property ColorPercentageS: SmallInt read FColorRatioS write SetColorRatioS default 100;
    property ColorPercentageV: SmallInt read FColorRatioV write SetColorRatioV default 100;
    property InvertColors: Boolean read FInvertColors write SetInvertColors default False;
    property GrayScale: Boolean read FGrayScale write SetGrayScale default False;
    property Masked: Boolean read FMasked write SetMasked default False;
    property MaskColor: TColor read FMaskColor write SetMaskColor default clGrayText;
    property CustomColoring: Boolean read FCustomColoring write SetCustomColoring default False;
    property WidthPercentage: SmallInt read FWidthRatio write SetWidthRatio default 100;
    property HeightPercentage: SmallInt read FHeightRatio write SetHeightRatio default 100;
    property RotationAngle: SmallInt read FRotationAngle write SetRotationAngle default 0;
    property CustomMapping: Boolean read FCustomMapping write SetCustomMapping default False;
    property ColorInversionR: Byte read FColorInversionR write SetColorInversionR default 0;
    property ColorInversionG: Byte read FColorInversionG write SetColorInversionG default 0;
    property ColorInversionB: Byte read FColorInversionB write SetColorInversionB default 0;
    property AnimateFromLastState: Boolean read FAnimateFromLastState write FAnimateFromLastState default True;
    property AnimationInterval: Cardinal read FAnimationInterval write FAnimationInterval default 50;
    property AnimationDuration: Cardinal read FFAnimationDuration write FFAnimationDuration default 200;
    property AnimationScript: TStrings read FAnimationScript write SetAnimationScript;
    property AnimatedEffect: Boolean read FAnimatedEffect write SetAnimatedEffect default False;
    property ReplayAnimation: Boolean read FReplayAnimation write SetReplayAnimation default True;
    property Brightness: SmallInt read FBrightness write SetBrightness default 0;
    property Contrast: SmallInt read FContrast write SetContrast default 0;
    property Shadow: Boolean read FShadow write SetShadow default False;
    property ShadowColor: TColor read FShadowColor write SetShadowColor default clBlack;
    property ShadowX: Integer read FShadowX write SetShadowX default 2;
    property ShadowY: Integer read FShadowY write SetShadowY default 2;
    property ShadowAlpha: Byte read FShadowAlpha write SetShadowAlpha default 100;
    property ShadowBlurRadius: Byte read FShadowBlurRadius write SetShadowBlurRadius default 2;
    property OverlayImage: Boolean read FOverlayImage write SetOverlayImage default False;
    property MoveX: Integer read FMoveX write SetMoveX default 0;
    property MoveY: Integer read FMoveY write SetMoveY default 0;
  end;

  TdihavButton = class(TCustomControl)
  private
    FCancel: Boolean;
    FDefault: Boolean;
    FActive: Boolean;
    FModalResult: TModalResult;
    FButtonState: TdihavButtonState;
    FClicksDisabled: Boolean;
    FDown: Boolean;
    FGroupIndex: Integer;
    FAllowAllUp: Boolean;
    FImageList: TCustomImageList;
    FImageChangeLink: TChangeLink;
    FImageIndex: TImageIndex;
    FPicture: TPicture;
    FEffectNormal: TdihavButtonEffect;
    FEffectOver: TdihavButtonEffect;
    FEffectDown: TdihavButtonEffect;
    FEffectFocused: TdihavButtonEffect;
    FEffectDisabled: TdihavButtonEffect;
    FEffectActive: TdihavButtonEffect;
    FCurrentEffect: TdihavButtonEffect;
    FConsiderRotationSize: Boolean;
    FImageShape: Boolean;
    FCaptionAlignment: TAlignment;
    FCaptionLayout: TTextLayout;
    FCustomBackground: Boolean;
    FCustomBackgroundEvent: TdihavButtonCustomBackground;
    FCustomAnimationScriptEvent: TdihavButtonCustomAnimationScript;
    FButtonStateChangedEvent: TdihavButtonStateChanged;
    FCheckButtonState, FAutoCheckButtonState: Boolean;

    function GetHomePage: String;
    procedure SetHomePage(Value: String);
    procedure SetDefault(Value: Boolean);
    procedure CMDialogKey(var Message: TCMDialogKey); message CM_DIALOGKEY;
    procedure CMFocusChanged(var Message: TCMFocusChanged); message CM_FOCUSCHANGED;
    procedure CNCommand(var Message: TWMCommand); message CN_COMMAND;
    procedure SetButtonState(Value: TdihavButtonState);
    procedure SetDown(Value: Boolean);
    procedure SetAllowAllUp(Value: Boolean);
    procedure SetGroupIndex(Value: Integer);
    procedure UpdateExclusive;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMButtonPressed(var Message: TMessage); message CM_BUTTONPRESSED;
    procedure CMSysColorChange(var Message: TMessage); message CM_SYSCOLORCHANGE;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMGotFocus(var Message: TMessage); message CM_GOTFOCUS;
    procedure CMLostFocus(var Message: TMessage); message CM_LOSTFOCUS;
    procedure CMColorChanged(var Message:TMessage); message CM_COLORCHANGED;
    procedure WMKeyUp(var Message: TWMKeyUp); message WM_KEYUP;
    procedure WMKeyDown(var Message: TWMKeyDown); message WM_KEYDOWN;
    procedure SetImageList(Value: TCustomImageList);
    procedure ImgChange(Sender: TObject);
    procedure GraphsChanged;
    procedure SetImageIndex(Value: TImageIndex);
    procedure SetPicture(Value: TPicture);
    procedure SetConsiderRotationSize(Value: Boolean);
    procedure TimerTick(Sender: TObject);
    procedure SetImageShape(Value: Boolean);
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure CMBiDiModeChanged(var Message: TMessage); message CM_BIDIMODECHANGED;
    procedure SetCaptionAlignment(Value: TAlignment);
    procedure SetCaptionLayout(Value: TTextLayout);
    procedure SetCustomBackground(Value: Boolean);
    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure WMSetFocus(var Message:TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus(var Message:TWMKillFocus); message WM_KILLFOCUS;
    procedure SetCheckButtonState(Value: Boolean);
    function CheckButtonStateStored: Boolean;
    procedure SetAutoCheckButtonState(Value: Boolean);
    procedure UpdateStateBackImagesNoImgChange;
  protected
    FCustomMapping: TdihavButtonCustomMapping;
    FCustomColoring: TdihavButtonCustomColoring;
    FTimer: TTimer;
    FAnimationPos: Real;
    FTickHolder: Cardinal;
    FRGN: HRGN;
    FNormalImgR, FNormalImgB, FDownImgR, FDownImgB: TBitmap;
    FAssigningEffects: Boolean;

    FColorShiftR, FColorShiftG, FColorShiftB, FColorShiftA: SmallInt;
    FColorRatioR, FColorRatioG, FColorRatioB, FColorRatioA: SmallInt;
    FColorShiftH, FColorShiftS, FColorShiftV: SmallInt;
    FColorRatioH, FColorRatioS, FColorRatioV: SmallInt;
    FInvertColors: Boolean;
    FGrayScale: Boolean;
    FMasked: Boolean;
    FMaskColor: TColor;
    FWidthRatio, FHeightRatio: SmallInt;
    FRotationAngle: SmallInt;
    FColorInversionR, FColorInversionG, FColorInversionB: Byte;
    FBrightness, FContrast: SmallInt;
    FShadow: Boolean;
    FShadowColor: TColor;
    FShadowX, FShadowY: Integer;
    FShadowAlpha: Byte;
    FShadowBlurRadius: Byte;
    FMoveX, FMoveY: Integer;

    _FColorShiftR, _FColorShiftG, _FColorShiftB, _FColorShiftA: SmallInt;
    _FColorRatioR, _FColorRatioG, _FColorRatioB, _FColorRatioA: SmallInt;
    _FColorShiftH, _FColorShiftS, _FColorShiftV: SmallInt;
    _FColorRatioH, _FColorRatioS, _FColorRatioV: SmallInt;
    _FInvertColors: Boolean;
    _FGrayScale: Boolean;
    _FMasked: Boolean;
    _FMaskColor: TColor;
    _FWidthRatio, _FHeightRatio: SmallInt;
    _FRotationAngle: SmallInt;
    _FColorInversionR, _FColorInversionG, _FColorInversionB: Byte;
    _FBrightness, _FContrast: SmallInt;
    _FShadow: Boolean;
    _FShadowColor: TColor;
    _FShadowX, _FShadowY: Integer;
    _FShadowAlpha: Byte;
    _FShadowBlurRadius: Byte;
    _FMoveX, _FMoveY: Integer;

    FColorShiftRD, FColorShiftGD, FColorShiftBD, FColorShiftAD: Real;
    FColorRatioRD, FColorRatioGD, FColorRatioBD, FColorRatioAD: Real;
    FColorShiftHD, FColorShiftSD, FColorShiftVD: Real;
    FColorRatioHD, FColorRatioSD, FColorRatioVD: Real;
    FInvertColorsD: Boolean;
    FGrayScaleD: Boolean;
    FMaskedD: Boolean;
    FMaskColorRD, FMaskColorGD, FMaskColorBD: Real;
    FWidthRatioD, FHeightRatioD: Real;
    FRotationAngleD: Real;
    FColorInversionRD, FColorInversionGD, FColorInversionBD: Real;
    FBrightnessD, FContrastD: Real;
    FShadowColorRD, FShadowColorGD, FShadowColorBD: Real;
    FShadowXD, FShadowYD: Real;
    FShadowAlphaD: Real;
    FShadowBlurRadiusD: Real;
    FMoveXD, FMoveYD: Real;

    FColorShiftR0, FColorShiftG0, FColorShiftB0, FColorShiftA0: SmallInt;
    FColorRatioR0, FColorRatioG0, FColorRatioB0, FColorRatioA0: SmallInt;
    FColorShiftH0, FColorShiftS0, FColorShiftV0: SmallInt;
    FColorRatioH0, FColorRatioS0, FColorRatioV0: SmallInt;
    FMaskColorR0, FMaskColorG0, FMaskColorB0 : Byte;
    FWidthRatio0, FHeightRatio0: SmallInt;
    FRotationAngle0: SmallInt;
    FColorInversionR0, FColorInversionG0, FColorInversionB0: Byte;
    FBrightness0, FContrast0: SmallInt;
    FShadowColorR0, FShadowColorG0, FShadowColorB0: Byte;
    FShadowX0, FShadowY0: Integer;
    FShadowAlpha0: Byte;
    FShadowBlurRadius0: Byte;
    FMoveX0, FMoveY0: Integer;

    procedure Paint; override;
    procedure CreateWnd; override;
    procedure WndProc(var Message: TMessage); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure Loaded; override;
    procedure ActionChange(Sender: TObject; CheckDefaults: Boolean); override;
    function GetActionLinkClass: TControlActionLinkClass; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function CanAutoSize(var NewWidth, NewHeight: Integer): Boolean; override;
    procedure UpdateAutoSize;
    procedure SaveEffectVars;
    procedure RestoreEffectVars;
    procedure AssignEffectVars(AEffect: TdihavButtonEffect);
    procedure CalcEffectDelta(AEffect: TdihavButtonEffect);

    property ClicksDisabled: Boolean read FClicksDisabled write FClicksDisabled;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Click; override;
    procedure DblClick; override;
    procedure AssignEffectsFromButton(Source: TdihavButton);

    property ButtonState: TdihavButtonState read FButtonState write SetButtonState;
    property CurrentEffect: TdihavButtonEffect read FCurrentEffect;
    property AnimationPosition: Real read FAnimationPos;
    property RGN: HRGN read FRGN;
  published
    property AllowAllUp: Boolean read FAllowAllUp write SetAllowAllUp default False;
    property Cancel: Boolean read FCancel write FCancel default False;
    property Default: Boolean read FDefault write SetDefault default False;
    Property GroupIndex: Integer read FGroupIndex write SetGroupIndex default 0;
    property Down: Boolean read FDown write SetDown default False;
    property Homepage: String read GetHomePage write SetHomePage stored False;
    property Images: TCustomImageList read FImageList write SetImageList;
    property ImageIndex: TImageIndex read FImageIndex write SetImageIndex default -1;
    property ModalResult: TModalResult read FModalResult write FModalResult default 0;
    property Picture: TPicture read FPicture write SetPicture;
    property EffectNormal: TdihavButtonEffect read FEffectNormal write FEffectNormal;
    property EffectOver: TdihavButtonEffect read FEffectOver write FEffectOver;
    property EffectDown: TdihavButtonEffect read FEffectDown write FEffectDown;
    property EffectFocused: TdihavButtonEffect read FEffectFocused write FEffectFocused;
    property EffectDisabled: TdihavButtonEffect read FEffectDisabled write FEffectDisabled;
    property EffectActive: TdihavButtonEffect read FEffectActive write FEffectActive;
    property ConsiderRotationSize: Boolean read FConsiderRotationSize write SetConsiderRotationSize default True;
    property ImageShape: Boolean read FImageShape write SetImageShape default False;
    property CaptionAlignment: TAlignment read FCaptionAlignment write SetCaptionAlignment default taCenter;
    property CaptionLayout: TTextLayout read FCaptionLayout write SetCaptionLayout default tlCenter;
    property CustomBackground: Boolean read FCustomBackground write SetCustomBackground default False;
    property CheckButtonState: Boolean read FCheckButtonState write SetCheckButtonState stored CheckButtonStateStored;
    property AutoCheckButtonState: Boolean read FAutoCheckButtonState write SetAutoCheckButtonState default True;

    property OnCustomMapping: TdihavButtonCustomMapping read FCustomMapping write FCustomMapping;
    property OnCustomColoring: TdihavButtonCustomColoring read FCustomColoring write FCustomColoring;
    property OnCustomBackground: TdihavButtonCustomBackground read FCustomBackgroundEvent write FCustomBackgroundEvent;
    property OnCustomAnimationScript: TdihavButtonCustomAnimationScript read FCustomAnimationScriptEvent write FCustomAnimationScriptEvent;
    property OnStateChanged: TdihavButtonStateChanged read FButtonStateChangedEvent write FButtonStateChangedEvent;

    property Action;
    property Align;
    property AlignWithMargins;
    property Anchors;
    property AutoSize default True;
    property BiDiMode;
    property Caption;
    property Color;
    property Constraints;
    property Cursor;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property Height;
    property HelpContext;
    property HelpKeyword;
    property HelpType;
    property Hint;
    property Left;
    property Margins;
    property Name;
    property ParentBiDiMode;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop default True;
    property Tag;
    property Top;
    property Visible;
    property Width;

    property OnCanResize;
    property OnClick;
    property OnConstrainedResize;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseActivate;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnStartDock;
    property OnStartDrag;
  end;

  TdihavButtonActionLink = class(TControlActionLink)
  protected
    FClient: TdihavButton;
    procedure AssignClient(AClient: TObject); override;
    function IsCheckedLinked: Boolean; override;
    function IsGroupIndexLinked: Boolean; override;
    function IsImageIndexLinked: Boolean; override;
    procedure SetGroupIndex(Value: Integer); override;
    procedure SetChecked(Value: Boolean); override;
    procedure SetImageIndex(Value: Integer); override;
  public
    constructor Create(AClient: TObject); override;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('dihav', [TdihavButton]);
end;

function ToByte(X: Single): Byte;
begin
if (X <= 0) then
  Result := 0
else if (X >= 255) then
  Result := 255
else
  Result := Round(X);
end;

procedure HSV2RGB(H: Word; S, V: Byte; var R, G, B: Byte);
var
  p, q, t: Byte;
  f, j: Integer;
begin
if (H > 359) then H := 0;
if (S > 100) then S := 100;
if (V > 100) then V := 100;
V := ToByte(V * 2.55);
if (S = 0) then
  begin
  R := V;
  G := V;
  B := V;
  end
else
  begin
  f := h mod 60;
  j := h div 60;
  p := ToByte(V * (1 - S / 100));
  q := ToByte(V * (1 - (S * f) / 6000));
  t := ToByte(V * (1 - (S * (60 - f) / 6000)));
  case j of
    0:
      begin
      R := V;
      G := t;
      B := p;
      end;
    1:
      begin
      R := q;
      G := V;
      B := p;
      end;
    2:
      begin
      R := p;
      G := V;
      B := t;
      end;
    3:
      begin
      R := p;
      G := q;
      B := V;
      end;
    4:
      begin
      R := t;
      G := p;
      B := V;
      end;
    5:
      begin
      R := V;
      G := p;
      B := q;
      end;
  else
    begin
    R := 0;
    G := 0;
    B := 0;
    end;
  end;
  end;
end;

procedure RGB2HSV(R, G, B: Byte; var H: Word; var S, V: Byte);
var
  m, d, j: Integer;
begin
m := r;
if (g < m) then m := g;
if (b < m) then m := b;
V := r;
if (g > V) then V := g;
if (b > V) then V := b;
d := V - m;
if (V = 0) then
  S := 0
else
  S := MulDiv(d, 255, V);
if (S = 0) then
  j := 0
else
  begin
  if (R = V) then
    j := MulDiv(G - B, 60, d)
  else if (G = V) then
    j := 120 + MulDiv(B - R, 60, d)
  else
    j := 240 + MulDiv(R - G, 60, d);
  end;
if (j < 0) then j := j + 360;
H := j;
V := ToByte(V * 100 / 255);
S := ToByte(S * 100 / 255);
end;

// TdihavButtonEffect

constructor TdihavButtonEffect.Create(AOwner: TdihavButton; AType: TdihavButtonEffectType);
begin
inherited Create;
FOwner := AOwner;
FType := AType;
FPicture := TPicture.Create;
FPicture.OnChange := ImgChange;
FImageIndex := -1;
FImg := TBitmap.Create;
FImg.PixelFormat := pf24bit;
FImgMask := TBitmap.Create;
FImgMask.PixelFormat := pf8bit;
FColorShiftR := 0;
FColorShiftG := 0;
FColorShiftB := 0;
FColorShiftA := 0;
FColorRatioR := 100;
FColorRatioG := 100;
FColorRatioB := 100;
FColorRatioA := 100;
FColorShiftH := 0;
FColorShiftS := 0;
FColorShiftV := 0;
FColorRatioH := 100;
FColorRatioS := 100;
FColorRatioV := 100;
FInvertColors := False;
FGrayScale := False;
FMasked := False;
FMaskColor := clGrayText;
FCustomColoring := False;
FWidthRatio := 100;
FHeightRatio := 100;
FRotationAngle := 0;
FCustomMapping := False;
FMaxSize.cx := 0;
FMaxSize.cy := 0;
FColorInversionR := 0;
FColorInversionG := 0;
FColorInversionB := 0;
FAnimateFromLastState := True;
FAnimationInterval := 50;
FFAnimationDuration := 200;
FAnimationScript := TStringList.Create;;
FAnimatedEffect := False;
FReplayAnimation := True;
FAnimationScriptPos := 0;
FTimer := TTimer.Create(FOwner);
FTimer.Enabled := False;
FTimer.OnTimer := TimerTick;
FBrightness := 0;
FContrast := 0;
FShadow := False;
FShadowColor := clBlack;
FShadowX := 2;
FShadowY := 2;
FShadowAlpha := 100;
FShadowBlurRadius := 2;
FOverlayImage := False;
FMoveX := 0;
FMoveY := 0;
FApplyChangesNow := True;
FImgW := 0;
FImgH := 0;
end;

destructor TdihavButtonEffect.Destroy;
begin
FPicture.Free;
FImg.Free;
FImgMask.Free;
FAnimationScript.Free;
inherited Destroy;
end;

procedure TdihavButtonEffect.SetImageIndex(Value: TImageIndex);
begin
if (FImageIndex <> Value) then
  begin
  FImageIndex := Value;
  GraphChanged;
  end;
end;

procedure TdihavButtonEffect.SetPicture(Value: TPicture);
begin
FPicture.Assign(Value);
FPicture.OnChange := ImgChange;
GraphChanged;
end;

procedure TdihavButtonEffect.GraphChanged;
var
  r, b: TBitmap;
  x, y: Integer;
  br, bb, bi, bm: PByteArray;
  r1, r2, g1, g2, b1, b2, a: Byte;
  s: String;
  c, d: TRect;
  tf: TTextFormat;

  function ChkImgLst(II: Integer; IL: TCustomImageList): Boolean;
  begin
  Result := False;
  if (II < 0) then Exit;
  if not Assigned(IL) then Exit;
  if (II >= IL.Count) then Exit;
  Result := True;
  end;

  function ChkActImgLst(II: Integer): Boolean;
  begin
  Result := False;
  if (II < 0) then Exit;
  if not Assigned(FOwner.Action) then Exit;
  if not (FOwner.Action is TCustomAction) then Exit;
  if not Assigned(TCustomAction(FOwner.Action).ActionList.Images) then Exit;
  if (II >= TCustomAction(FOwner.Action).ActionList.Images.Count) then Exit;
  Result := True;
  end;

begin
r := TBitmap.Create;
r.Canvas.Pen.Color := clRed;
r.Canvas.Brush.Color := clRed;
b := TBitmap.Create;
b.Canvas.Pen.Color := clBlue;
b.Canvas.Brush.Color := clBlue;
if ((FPicture.Width <> 0) and (FPicture.Height <> 0) and (not FOverlayImage)) then
  begin
  r.Width := FPicture.Width;
  r.Height := FPicture.Height;
  b.Width := r.Width;
  b.Height := r.Height;
  r.Canvas.FillRect(r.Canvas.ClipRect);
  b.Canvas.FillRect(b.Canvas.ClipRect);
  r.Canvas.Draw(0, 0, FPicture.Graphic);
  b.Canvas.Draw(0, 0, FPicture.Graphic);
  end
else if (ChkImgLst(FImageIndex, FOwner.Images) and (not FOverlayImage)) then
  begin
  r.Width := FOwner.Images.Width;
  r.Height := FOwner.Images.Height;
  b.Width := r.Width;
  b.Height := r.Height;
  r.Canvas.FillRect(r.Canvas.ClipRect);
  b.Canvas.FillRect(b.Canvas.ClipRect);
  FOwner.Images.Draw(r.Canvas, 0, 0, FImageIndex);
  FOwner.Images.Draw(b.Canvas, 0, 0, FImageIndex);
  end
else if (ChkActImgLst(FImageIndex) and (not FOverlayImage)) then
  begin
  r.Width := TCustomAction(FOwner.Action).ActionList.Images.Width;
  r.Height := TCustomAction(FOwner.Action).ActionList.Images.Height;
  b.Width := r.Width;
  b.Height := r.Height;
  r.Canvas.FillRect(r.Canvas.ClipRect);
  b.Canvas.FillRect(b.Canvas.ClipRect);
  TCustomAction(FOwner.Action).ActionList.Images.Draw(r.Canvas, 0, 0, FImageIndex);
  TCustomAction(FOwner.Action).ActionList.Images.Draw(b.Canvas, 0, 0, FImageIndex);
  end
else if ((FOwner.Picture.Width <> 0) and (FOwner.Picture.Height <> 0)) then
  begin
  r.Width := FOwner.Picture.Width;
  r.Height := FOwner.Picture.Height;
  b.Width := r.Width;
  b.Height := r.Height;
  r.Canvas.FillRect(r.Canvas.ClipRect);
  b.Canvas.FillRect(b.Canvas.ClipRect);
  r.Canvas.Draw(0, 0, FOwner.Picture.Graphic);
  b.Canvas.Draw(0, 0, FOwner.Picture.Graphic);
  end
else if (ChkImgLst(FOwner.ImageIndex, FOwner.Images)) then
  begin
  r.Width := FOwner.Images.Width;
  r.Height := FOwner.Images.Height;
  b.Width := r.Width;
  b.Height := r.Height;
  r.Canvas.FillRect(r.Canvas.ClipRect);
  b.Canvas.FillRect(b.Canvas.ClipRect);
  FOwner.Images.Draw(r.Canvas, 0, 0, FOwner.ImageIndex);
  FOwner.Images.Draw(b.Canvas, 0, 0, FOwner.ImageIndex);
  end
else if (ChkActImgLst(FOwner.ImageIndex)) then
  begin
  r.Width := TCustomAction(FOwner.Action).ActionList.Images.Width;
  r.Height := TCustomAction(FOwner.Action).ActionList.Images.Height;
  b.Width := r.Width;
  b.Height := r.Height;
  r.Canvas.FillRect(r.Canvas.ClipRect);
  b.Canvas.FillRect(b.Canvas.ClipRect);
  TCustomAction(FOwner.Action).ActionList.Images.Draw(r.Canvas, 0, 0, FOwner.ImageIndex);
  TCustomAction(FOwner.Action).ActionList.Images.Draw(b.Canvas, 0, 0, FOwner.ImageIndex);
  end;
if (FOverlayImage) then
  begin
  if ((FPicture.Width <> 0) and (FPicture.Height <> 0)) then
    begin
    r.Canvas.Draw(0, 0, FPicture.Graphic);
    b.Canvas.Draw(0, 0, FPicture.Graphic);
    end
  else if ChkImgLst(FImageIndex, FOwner.Images) then
    begin
    FOwner.Images.Draw(r.Canvas, 0, 0, FImageIndex);
    FOwner.Images.Draw(b.Canvas, 0, 0, FImageIndex);
    end
  else if ChkActImgLst(FImageIndex) then
    begin
    TCustomAction(FOwner.Action).ActionList.Images.Draw(r.Canvas, 0, 0, FImageIndex);
    TCustomAction(FOwner.Action).ActionList.Images.Draw(b.Canvas, 0, 0, FImageIndex);
    end;
  end;
FImgW := r.Width;
FImgH := r.Height;
s := FOwner.Caption;
if ((FType <> etNormal) and (FType <> etDown) and (FOwner.FCheckButtonState)) then
  begin
  if (FOwner.FDown) then
    begin
    r.Assign(FOwner.FDownImgR);
    b.Assign(FOwner.FDownImgB);
    end
  else
    begin
    r.Assign(FOwner.FNormalImgR);
    b.Assign(FOwner.FNormalImgB);
    end;
  if ((FPicture.Width <> 0) and (FPicture.Height <> 0)) then
    begin
    x := (r.Width - FPicture.Width) div 2;
    y := (r.Height - FPicture.Height) div 2;
    r.Canvas.Draw(x, y, FPicture.Graphic);
    b.Canvas.Draw(x, y, FPicture.Graphic);
    end
  else if ChkImgLst(FImageIndex, FOwner.Images) then
    begin
    x := (r.Width - FOwner.Images.Width) div 2;
    y := (r.Height - FOwner.Images.Height) div 2;
    FOwner.Images.Draw(r.Canvas, x, y, FImageIndex);
    FOwner.Images.Draw(b.Canvas, x, y, FImageIndex);
    end
  else if ChkActImgLst(FImageIndex) then
    begin
    x := (r.Width - TCustomAction(FOwner.Action).ActionList.Images.Width) div 2;
    y := (r.Height - TCustomAction(FOwner.Action).ActionList.Images.Height) div 2;
    TCustomAction(FOwner.Action).ActionList.Images.Draw(r.Canvas, x, y, FImageIndex);
    TCustomAction(FOwner.Action).ActionList.Images.Draw(b.Canvas, x, y, FImageIndex);
    end;
  s := '';
  end;
FImg.Width := r.Width;
FImg.Height := r.Height;
FImgMask.Width := r.Width;
FImgMask.Height := r.Height;
if (s <> '') then
  begin
  case FOwner.CaptionAlignment of
    taLeftJustify: tf := [tfLeft];
    taRightJustify: tf := [tfRight];
    taCenter: tf := [tfCenter];
  end;
  case FOwner.CaptionLayout of
    tlTop: Include(tf, tfTop);
    tlCenter: Include(tf, tfVerticalCenter);
    tlBottom: Include(tf, tfBottom);
  end;
  if (FOwner.BiDiMode <> bdLeftToRight) then Include(tf, tfRtlReading);
  r.Canvas.Brush.Style := bsClear;
  b.Canvas.Brush.Style := bsClear;
  r.Canvas.Font.Assign(FOwner.Font);
  b.Canvas.Font.Assign(FOwner.Font);
  c := r.Canvas.ClipRect;
  if (Pos(#13, s) <> 0) then
    begin
    d := c;
    y := DrawText(r.Canvas.Handle, PChar(s), Length(s) + 1, d, DT_CALCRECT);
    if (FOwner.CaptionLayout = tlBottom) then
      c.Top := c.Bottom - y;
    if (FOwner.CaptionLayout = tlCenter) then
      begin
      y := (c.Bottom - c.Top - y) div 2;
      c.Top := c.Top + y;
      c.Bottom := c.Bottom - y;
      end;
    end
  else
    Include(tf, tfSingleLine);
  r.Canvas.TextRect(c, s, tf);
  b.Canvas.TextRect(c, s, tf);
  end;
r.PixelFormat := pf24bit;
b.PixelFormat := pf24bit;
FImg.PixelFormat := pf24bit;
FImgMask.PixelFormat := pf8bit;
for y := r.Height - 1 downto 0 do
  begin
  br := r.ScanLine[y];
  bb := b.ScanLine[y];
  bi := FImg.ScanLine[y];
  bm := FImgMask.ScanLine[y];
  for x := 0 to r.Width - 1 do
    begin
    b1 := br^[x * 3];
    g1 := br^[x * 3 + 1];
    r1 := br^[x * 3 + 2];
    b2 := bb^[x * 3];
    g2 := bb^[x * 3 + 1];
    r2 := bb^[x * 3 + 2];
    if (RGB(r1, g1, b1) = RGB(r2, b2, g2)) then
      a := 255
    else if ((r1 = 255) and (b2 = 255) and (g1 + b1 + r2 + g2 = 0)) then
      a := 0
    else
      begin
      a := r2 - r1 + 255;
      if (a <> 0) then
       begin
        r2 := r2 * 255 div a;
        g2 := g2 * 255 div a;
        b2 := b1 * 255 div a;
        end;
      end;
    bi^[x * 3] := b2;
    bi^[x * 3 + 1] := g2;
    bi^[x * 3 + 2] := r2;
    bm^[x] := a;
    end;
  end;
r.Free;
b.Free;
UpdateMaxSize;
PropertyChanged;
end;

procedure TdihavButtonEffect.Draw(Bitmap: TBitmap);
var
  r, g, b, a: Byte;
  fx, fy, tx, ty: Single;
  cy: Integer;
  bi, bm: PByteArray;

  procedure GP(AX, AY: Integer);
  begin
  if ((AX < 0) or (AX >= FImg.Width) or (AY < 0) or (AY >= FImg.Height)) then
    begin
    r := 0;
    g := 0;
    b := 0;
    a := 0;
    Exit;
    end;
  if (cy <> AY) then
    begin
    bi := FImg.ScanLine[AY];
    bm := FImgMask.ScanLine[AY];
    cy := AY;
    end;
  b := bi^[AX * 3];
  g := bi^[AX * 3 + 1];
  r := bi^[AX * 3 + 2];
  a := bm^[AX];
  end;

  procedure GPF(XX, YY: Single);
  var
    a00, a01, a10, a11, ax, ay: Single;
    tx, ty: Integer;
    r00, r10, r01, r11: Byte;
    g00, g10, g01, g11: Byte;
    b00, b10, b01, b11: Byte;
    l00, l10, l01, l11: Byte;
  begin
  r := 0;
  g := 0;
  b := 0;
  a := 0;
  if (XX < -1) or (XX >= FImg.Width) or (YY < -1) or (YY >= FImg.Height) then Exit;
  if (XX < 0) then
    tx := -1
  else
    tx := Trunc(XX);
  if (YY < 0) then
    ty := -1
  else
    ty := Trunc(YY);
  ax := XX - tx;
  ay := YY - ty;
  a00 := (1 - ax) * (1 - ay);
  a01 := (1 - ax) * ay;
  a10 := ax * (1 - ay);
  a11 := ax * ay;
  GP(tx, ty);
  r00 := r;
  g00 := g;
  b00 := b;
  l00 := a;
  GP(tx + 1, ty);
  r10 := r;
  g10 := g;
  b10 := b;
  l10 := a;
  GP(tx, ty + 1);
  r01 := r;
  g01 := g;
  b01 := b;
  l01 := a;
  GP(tx + 1, ty + 1);
  r11 := r;
  g11 := g;
  b11 := b;
  l11 := a;
  a := ToByte(l00 * a00 + l10 * a10 + l01 * a01 + l11 * a11);
  if (a <> 0) then
    begin
    r := ToByte((r00 * l00 * a00 + r10 * l10 * a10 + r01 * l01 * a01 + r11 * l11 * a11) / a);
    g := ToByte((g00 * l00 * a00 + g10 * l10 * a10 + g01 * l01 * a01 + g11 * l11 * a11) / a);
    b := ToByte((b00 * l00 * a00 + b10 * l10 * a10 + b01 * l01 * a01 + b11 * l11 * a11) / a);
    end
  else
    begin
    r := ToByte((r00 * a00 + r10 * a10 + r01 * a01 + r11 * a11));
    g := ToByte((g00 * a00 + g10 * a10 + g01 * a01 + g11 * a11));
    b := ToByte((b00 * a00 + b10 * a10 + b01 * a01 + b11 * a11));
    end;
  end;

  procedure GR;

    procedure GL(Y: Integer);
    var x, n, as_, rs, gs, bs, rsp, gsp, bsp, ns: Single;
    begin
    x := fx;
    as_ := 0;
    rs := 0;
    gs := 0;
    bs := 0;
    rsp := 0;
    gsp := 0;
    bsp := 0;
    ns := 0;
    while (x <> tx) do
      begin
      GPF(Int(x), Y);
      if (Int(x) + 1 > tx) then
        begin
        n := tx - x;
        x := tx;
        end
      else
        begin
        n := 1 - Frac(x);
        x := Int(x) + 1;
        end;
      as_ := as_ + a * n;
      rs := rs + a * r * n / 255;
      gs := gs + a * g * n / 255;
      bs := bs + a * b * n / 255;
      rsp := rsp + r * n;
      gsp := gsp + g * n;
      bsp := bsp + b * n;
      ns := ns + n;
      end;
    if as_ = 0 then
      begin
      r := ToByte(rsp / ns);
      g := ToByte(gsp / ns);
      b := ToByte(bsp / ns);
      a := 0;
      end
    else
      begin
      r := ToByte(rs * 255 / as_);
      g := ToByte(gs * 255 / as_);
      b := ToByte(bs * 255 / as_);
      a := ToByte(as_ / (tx - fx));
      end;
    end;

  var y, m, az, rz, gz, bz, rzp, gzp, bzp, mz: Single;
  begin
  if (tx < fx) then
    begin
    y := fx;
    fx := tx;
    tx := y;
    end;
  if (ty < fy) then
    begin
    y := fy;
    fy := ty;
    ty := y;
    end;
  y := fy;
  az := 0;
  rz := 0;
  gz := 0;
  bz := 0;
  rzp := 0;
  gzp := 0;
  bzp := 0;
  mz := 0;
  while (y <> ty) do
    begin
    GL(Trunc(y));
    if (Int(y) + 1 > ty) then
      begin
      m := ty - y;
      y := ty;
      end
    else
      begin
      m := 1 - Frac(y);
      y := Int(y) + 1;
      end;
    az := az + a *m;
    rz := rz + a * r * m / 255;
    gz := gz + a * g * m / 255;
    bz := bz + a * b * m / 255;
    rzp := rzp + r * m;
    gzp := gzp + g * m;
    bzp := bzp + b * m;
    mz := mz + m;
    end;
  if az=0 then
    begin
    r := ToByte(rzp / mz);
    g := ToByte(gzp / mz);
    b := ToByte(bzp / mz);
    a := 0;
    end
  else
    begin
    r := ToByte(rz * 255 / az);
    g := ToByte(gz * 255 / az);
    b := ToByte(bz * 255 / az);
    a := ToByte(az / (ty - fy));
    end;
  end;

  procedure GPM;
  begin
  if ((fx = tx) or (fy = ty)) then
    GPF(fx, fy)
  else
    GR;
  end;

var
  x, y, k, dx, dy, j, i: Integer;
  t: PByteArray;
  v, s, q: Byte;
  h: Word;
  c: TColor;
  o, p: HRGN;
  n, u: Single;
  si: PByteArray;
  sa: PWordArray;
begin
cy := -1;
o := 0;
dx := Round((Bitmap.Width - FImg.Width * FOwner.FWidthRatio / 100) / 2);
dy := Round((Bitmap.Height - FImg.Height * FOwner.FHeightRatio / 100) / 2);
si := nil;
sa := nil;
t := nil;
if ((FOwner.FShadowAlpha = 0) and (FOwner.FShadow)) then
  begin
  dx := dx - FOwner.FShadowX;
  dy := dy - FOwner.FShadowY;
  end;
if (FOwner.FShadowAlpha <> 0) then
  begin
  si := AllocMem(Bitmap.Height * Bitmap.Width * 5);
  FillMemory(si, Bitmap.Height * Bitmap.Width * 5, 0);
  sa := AllocMem(Bitmap.Height * Bitmap.Width * 2);
  FillMemory(sa, Bitmap.Height * Bitmap.Width * 2, 0);
  end;
for y := Bitmap.Height - 1 downto 0 do
  begin
  if (FOwner.FShadowAlpha = 0) then t := Bitmap.ScanLine[y];
  for x := 0 to Bitmap.Width - 1 do
    begin
    if ((FOwner.FWidthRatio <> 100) or (FOwner.FHeightRatio <> 100) or (FOwner.FRotationAngle <> 0) or (FCustomMapping)) then
      begin
      fx := x - Bitmap.Width / 2;
      fy := y - Bitmap.Height / 2;
      if (FOwner.FRotationAngle <> 0) then
        begin
        n := FOwner.FRotationAngle * Pi / 1800;
        u :=  fx * Cos(n) - fy * Sin(n);
        fy := fx * Sin(n) + fy * Cos(n);
        fx := u;
        end;
      tx := fx + 1;
      ty := fy + 1;
      if ((FOwner.FWidthRatio <> 100) and (FOwner.FWidthRatio <> 0)) then
        begin
        fx := fx * 100 / FOwner.FWidthRatio;
        tx := tx * 100 / FOwner.FWidthRatio;
        end;
      if ((FOwner.FHeightRatio <> 100) and (FOwner.FHeightRatio <> 0)) then
        begin
        fy := fy * 100 / FOwner.FHeightRatio;
        ty := ty * 100 / FOwner.FHeightRatio;
        end;
      if FCustomMapping then
        begin
        CustomMap(fx, fy);
        CustomMap(tx, ty);
        end;
      fx := fx + FImg.Width / 2 - FOwner.FMoveX;
      fy := fy + FImg.Height / 2 - FOwner.FMoveY;
      tx := tx + FImg.Width / 2 - FOwner.FMoveX;
      ty := ty + FImg.Height / 2 - FOwner.FMoveY;
      GPM;
      end
    else
      GP(x - dx - FOwner.FMoveX, y - dy - FOwner.FMoveY);
    if ((FOwner.FColorRatioR <> 100) or (FOwner.FColorShiftR <> 0)) then
      r := ToByte(r * FOwner.FColorRatioR / 100 + FOwner.FColorShiftR);
    if ((FOwner.FColorRatioG <> 100) or (FOwner.FColorShiftG <> 0)) then
      g := ToByte(g * FOwner.FColorRatioG / 100 + FOwner.FColorShiftG);
    if ((FOwner.FColorRatioB <> 100) or (FOwner.FColorShiftB <> 0)) then
      b := ToByte(b * FOwner.FColorRatioB / 100 + FOwner.FColorShiftB);
    if ((FOwner.FColorRatioA <> 100) or (FOwner.FColorShiftA <> 0)) then
      a := ToByte(a * FOwner.FColorRatioA / 100 + FOwner.FColorShiftA);
    if ((FOwner.FColorRatioH <> 100) or (FOwner.FColorRatioS <> 100) or (FOwner.FColorRatioV <> 100) or
        (FOwner.FColorShiftH <> 0) or (FOwner.FColorShiftS <> 0) or (FOwner.FColorShiftV <> 0)) then
      begin
      RGB2HSV(r, g, b, h, s, v);
      k := Trunc(h * FOwner.FColorRatioH / 100 + FOwner.FColorShiftH);
      while (k < 0) do Inc(k, 360);
      while (k > 359) do Dec(k, 360);
      h := k;
      s := ToByte(s * FOwner.FColorRatioS / 100 + FOwner.FColorShiftS);
      v := ToByte(v * FOwner.FColorRatioV / 100 + FOwner.FColorShiftV);
      HSV2RGB(h, s, v, r, g, b);
      end;
    if (FOwner.FColorInversionR <> 0) then
      r := ToByte((r - 127.5) * (255 - 2 * FOwner.FColorInversionR) / 255 + 127.5);
    if (FOwner.FColorInversionG <> 0) then
      g := ToByte((g - 127.5) * (255 - 2 * FOwner.FColorInversionG) / 255 + 127.5);
    if (FOwner.FColorInversionB <> 0) then
      b := ToByte((b - 127.5) * (255 - 2 * FOwner.FColorInversionB) / 255 + 127.5);
    if ((FOwner.FInvertColorsD) and (FOwner.FTimer.Enabled)) then
      begin
      if FOwner.FInvertColors then
        begin
        r := ToByte((2 * r - 255) * FOwner.AnimationPosition + 255 - r);
        g := ToByte((2 * g - 255) * FOwner.AnimationPosition + 255 - g);
        b := ToByte((2 * b - 255) * FOwner.AnimationPosition + 255 - b);
        end
      else
        begin
        r := ToByte((255 - 2 * r) * FOwner.AnimationPosition + r);
        g := ToByte((255 - 2 * g) * FOwner.AnimationPosition + g);
        b := ToByte((255 - 2 * b) * FOwner.AnimationPosition + b);
        end;
      end
    else
      if FOwner.FInvertColors then
        begin
        r := 255 - r;
        g := 255 - g;
        b := 255 - b;
        end;
    if ((FOwner.FGrayScaleD) and (FOwner.FTimer.Enabled)) then
      begin
      q := ToByte(r * 0.3 + g * 0.59 + b * 0.11);
      if FOwner.FGrayScale then
        begin
        r := ToByte((r - q) * FOwner.AnimationPosition + q);
        g := ToByte((g - q) * FOwner.AnimationPosition + q);
        b := ToByte((b - q) * FOwner.AnimationPosition + q);
        end
      else
        begin
        r := ToByte((q - r) * FOwner.AnimationPosition + r);
        g := ToByte((q - g) * FOwner.AnimationPosition + g);
        b := ToByte((q - b) * FOwner.AnimationPosition + b);
        end;
      end
    else
      if FOwner.FGrayScale then
        begin
        r := ToByte(r * 0.3 + g * 0.59 + b * 0.11);
        g := r;
        b := r;
        end;
    if ((FOwner.FMaskedD) and (FOwner.FTimer.Enabled)) then
      begin
      c := ColorToRGB(FOwner.FMaskColor);
      if FOwner.FMasked then
        begin
        r := ToByte((r - GetRValue(c)) * FOwner.AnimationPosition + GetRValue(c));
        g := ToByte((g - GetGValue(c)) * FOwner.AnimationPosition + GetGValue(c));
        b := ToByte((b - GetBValue(c)) * FOwner.AnimationPosition + GetBValue(c));
        end
      else
        begin
        r := ToByte((GetRValue(c) - r) * FOwner.AnimationPosition + r);
        g := ToByte((GetGValue(c) - g) * FOwner.AnimationPosition + g);
        b := ToByte((GetBValue(c) - b) * FOwner.AnimationPosition + b);
        end;
      end
    else
      if FOwner.FMasked then
        begin
        c := ColorToRGB(FOwner.FMaskColor);
        r := GetRValue(c);
        g := GetGValue(c);
        b := GetBValue(c);
        end;
    if (FOwner.FBrightness <> 0) then
      begin
      r := ToByte(r + (255 - r) * FOwner.FBrightness / 255);
      g := ToByte(g + (255 - g) * FOwner.FBrightness / 255);
      b := ToByte(b + (255 - b) * FOwner.FBrightness / 255);
      end;
    if (FOwner.FContrast <> 0) then
      begin
      r := ToByte(r + (r - 127) * FOwner.FContrast / 255);
      g := ToByte(g + (g - 127) * FOwner.FContrast / 255);
      b := ToByte(b + (b - 127) * FOwner.FContrast / 255);
      end;
    if (FCustomColoring and Assigned(FOwner.FCustomColoring)) then
      FOwner.FCustomColoring(FOwner, Self, r, g, b, a);

    if (FOwner.FShadowAlpha <> 0) then
      begin
      j := (y * Bitmap.Width + x) * 5;
      si^[j] := r;
      si^[j + 1] := g;
      si^[j + 2] := b;
      si^[j + 3] := a;
      q := ToByte(a * FOwner.FShadowAlpha / 255);
      for i := x - FOwner.FShadowBlurRadius to x + FOwner.FShadowBlurRadius do
        for j := y - FOwner.FShadowBlurRadius to y + FOwner.FShadowBlurRadius do  
          if (i >= 0) and (j >= 0) and (i < Bitmap.Width) and (j < Bitmap.Height) then
            begin
            u := sqr(FOwner.FShadowBlurRadius) - sqr(i - x) - sqr(j - y); 
            if (u >= 0) then
              begin
              si^[(j * Bitmap.Width + i) * 5 + 4] := si^[(j * Bitmap.Width + i) * 5 + 4] + 1;
              sa^[j * Bitmap.Width + i] := sa^[j * Bitmap.Width + i] + q;
              end
            else if (u > -1) then
              begin
              si^[(j * Bitmap.Width + i) * 5 + 4] := si^[(j * Bitmap.Width + i) * 5 + 4] + 1;
              sa^[j * Bitmap.Width + i] := sa^[j * Bitmap.Width + i] + ToByte((1 + u) * q);
              end;
            end;
      end
    else
      begin
      j := x * 3;
      t^[j] := ToByte((t^[j] * (255 - a) + b * a) / 255);
      t^[j + 1] := ToByte((t^[j + 1] * (255 - a) + g * a) / 255);
      t^[j + 2] := ToByte((t^[j + 2] * (255 - a) + r * a) / 255);
      if (FOwner.ImageShape and (a <> 0)) then
        begin
        p := CreateRectRGN(x, y, x + 1, y + 1);
        if (o = 0) then
          o := p
        else
          begin
          CombineRGN(o, o, p, RGN_OR);
          DeleteObject(p);
          end;
        end;
      end;
    end;
  end;
if (FOwner.FShadowAlpha <> 0) then
  for y := Bitmap.Height - 1 downto 0 do
    begin
    t := Bitmap.ScanLine[y];
    for x := 0 to Bitmap.Width - 1 do
      begin
      r := 0;
      g := 0;
      b := 0;
      a := 0;
      u := 0;
      if (x + FOwner.FShadowX >= 0) and (x + FOwner.FShadowX < Bitmap.Width) and (y + FOwner.FShadowY >= 0) and (y + FOwner.FShadowY < Bitmap.Height) then  
        begin
        j := ((y + FOwner.FShadowY) * Bitmap.Width + x + FOwner.FShadowX) * 5; 
        r := si^[j];
        g := si^[j + 1];
        b := si^[j + 2];
        a := si^[j + 3];
        end;
      j := (y * Bitmap.Width + x) * 5;
      if (si^[j + 4] <> 0) then
        u := sa^[y * Bitmap.Width + x] / si^[j + 4];
      c := ColorToRGB(FOwner.FShadowColor);
      q := ToByte(a + u - u * a / 255);
      if (q <> 0) then
        begin
        r := ToByte((r * a + GetRValue(c) * u - GetRValue(c) * u * a / 255) / q);
        g := ToByte((g * a + GetGValue(c) * u - GetGValue(c) * u * a / 255) / q);
        b := ToByte((b * a + GetBValue(c) * u - GetBValue(c) * u * a / 255) / q);
        a := q;
        end
      else
        begin
        r := GetRValue(c);
        g := GetGValue(c);
        b := GetBValue(c);
        a := ToByte(u);
        end;
      j := x * 3;
      t^[j] := ToByte((t^[j] * (255 - a) + b * a) / 255);
      t^[j + 1] := ToByte((t^[j + 1] * (255 - a) + g * a) / 255);
      t^[j + 2] := ToByte((t^[j + 2] * (255 - a) + r * a) / 255);
      if (FOwner.ImageShape and (a <> 0)) then
        begin
        p := CreateRectRGN(x, y, x + 1, y + 1);
        if (o = 0) then
          o := p
        else
          begin
          CombineRGN(o, o, p, RGN_OR);
          DeleteObject(p);
          end;
        end;
      end;
    end;
if (si <> nil) then
  begin
  FreeMem(si);
  FreeMem(sa);
  end;
if (FOwner.ImageShape and (not EqualRgn(FOwner.FRGN, o)) and (not (csDesigning in FOwner.ComponentState))) then
  begin
  p := FOwner.FRGN;
  FOwner.FRGN := o;
  SetWindowRGN(FOwner.Handle, FOwner.FRGN, True);
  if (p <> 0) then DeleteObject(p);
  end
else
  if (o <> 0) then DeleteObject(o);
end;

procedure TdihavButtonEffect.ImgChange(Sender: TObject);
begin
GraphChanged;
end;

procedure TdihavButtonEffect.Repaint;
begin
if (FOwner.CurrentEffect = Self) then
  begin
  if ((FAnimateFromLastState) and (FOwner.AnimationPosition = 0)) then
    begin
    FOwner.CalcEffectDelta(Self);
    FOwner.FTimer.Interval := FAnimationInterval;
    FOwner.FTimer.Tag := 0;
    FOwner.FTimer.Enabled := True;
    FOwner.FTickHolder := GetTickCount;
    end
  else
    begin
    FOwner.FTimer.Enabled := False;
    FOwner.AssignEffectVars(Self);
    FOwner.Repaint;
    end;
  end;
end;

procedure TdihavButtonEffect.PropertyChanged;
begin
if (FOwner.CurrentEffect = Self) then
  begin
  FOwner.FAnimationPos := 0;
  if (FApplyChangesNow) then Repaint;
  end;
if ((FOwner.CheckButtonState) and (((FOwner.CurrentEffect = Self) and (FApplyChangesNow)) or (FOwner.CurrentEffect <> Self))) then
  SaveStateImages;
end;

procedure TdihavButtonEffect.SaveStateImages;
 var
  b: Boolean;
  r: Real;
begin
if (FType = etNormal) then
  begin
  FOwner.FNormalImgR.Width := FMaxSize.cx;
  FOwner.FNormalImgR.Height := FMaxSize.cy;
  FOwner.FNormalImgB.Width := FMaxSize.cx;
  FOwner.FNormalImgB.Height := FMaxSize.cy;
  FOwner.FNormalImgR.Canvas.FillRect(FOwner.FNormalImgR.Canvas.ClipRect);
  FOwner.FNormalImgB.Canvas.FillRect(FOwner.FNormalImgB.Canvas.ClipRect);
  b := FOwner.FTimer.Enabled;
  r := FOwner.FAnimationPos;
  FOwner.FTimer.Enabled := False;
  FOwner.FAnimationPos := 1;
  FOwner.SaveEffectVars;
  FOwner.AssignEffectVars(Self);
  Draw(FOwner.FNormalImgR);
  Draw(FOwner.FNormalImgB);
  FOwner.RestoreEffectVars;
  FOwner.FAnimationPos := r;
  FOwner.FTimer.Enabled := b;
  FOwner.EffectOver.GraphChanged;
  FOwner.EffectFocused.GraphChanged;
  FOwner.EffectDisabled.GraphChanged;
  FOwner.EffectActive.GraphChanged;
  end;
if (FType = etDown) then
  begin
  FOwner.FDownImgR.Width := FMaxSize.cx;
  FOwner.FDownImgR.Height := FMaxSize.cy;
  FOwner.FDownImgB.Width := FMaxSize.cx;
  FOwner.FDownImgB.Height := FMaxSize.cy;
  FOwner.FDownImgR.Canvas.FillRect(FOwner.FDownImgR.Canvas.ClipRect);
  FOwner.FDownImgB.Canvas.FillRect(FOwner.FDownImgB.Canvas.ClipRect);
  b := FOwner.FTimer.Enabled;
  r := FOwner.FAnimationPos;
  FOwner.FTimer.Enabled := False;
  FOwner.FAnimationPos := 1;
  FOwner.SaveEffectVars;
  FOwner.AssignEffectVars(Self);
  Draw(FOwner.FDownImgR);
  Draw(FOwner.FDownImgB);
  FOwner.RestoreEffectVars;
  FOwner.FAnimationPos := r;
  FOwner.FTimer.Enabled := b;
  FOwner.EffectOver.GraphChanged;
  FOwner.EffectFocused.GraphChanged;
  FOwner.EffectDisabled.GraphChanged;
  FOwner.EffectActive.GraphChanged;
  end;
end;

procedure TdihavButtonEffect.SetColorShiftR(Value: SmallInt);
begin
if (FColorShiftR <> Value) then
  begin
  FColorShiftR := Value;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetColorShiftG(Value: SmallInt);
begin
if (FColorShiftG <> Value) then
  begin
  FColorShiftG := Value;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetColorShiftB(Value: SmallInt);
begin
if (FColorShiftB <> Value) then
  begin
  FColorShiftB := Value;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetColorShiftA(Value: SmallInt);
begin
if (FColorShiftA <> Value) then
  begin
  FColorShiftA := Value;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetColorRatioR(Value: SmallInt);
begin
if (FColorRatioR <> Value) then
  begin
  FColorRatioR := Value;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetColorRatioG(Value: SmallInt);
begin
if (FColorRatioG <> Value) then
  begin
  FColorRatioG := Value;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetColorRatioB(Value: SmallInt);
begin
if (FColorRatioB <> Value) then
  begin
  FColorRatioB := Value;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetColorRatioA(Value: SmallInt);
begin
if (FColorRatioA <> Value) then
  begin
  FColorRatioA := Value;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetColorShiftH(Value: SmallInt);
begin
if (FColorShiftH <> Value) then
  begin
  FColorShiftH := Value;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetColorShiftS(Value: SmallInt);
begin
if (FColorShiftS <> Value) then
  begin
  FColorShiftS := Value;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetColorShiftV(Value: SmallInt);
begin
if (FColorShiftV <> Value) then
  begin
  FColorShiftV := Value;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetColorRatioH(Value: SmallInt);
begin
if (FColorRatioH <> Value) then
  begin
  FColorRatioH := Value;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetColorRatioS(Value: SmallInt);
begin
if (FColorRatioS <> Value) then
  begin
  FColorRatioS := Value;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetColorRatioV(Value: SmallInt);
begin
if (FColorRatioV <> Value) then
  begin
  FColorRatioV := Value;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetInvertColors(Value: Boolean);
begin
if (FInvertColors <> Value) then
  begin
  FInvertColors := Value;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetGrayScale(Value: Boolean);
begin
if (FGrayScale <> Value) then
  begin
  FGrayScale := Value;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetMasked(Value: Boolean);
begin
if (FMasked <> Value) then
  begin
  FMasked := Value;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetMaskColor(Value: TColor);
begin
if (FMaskColor <> Value) then
  begin
  FMaskColor := Value;
  if FMasked then PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetCustomColoring(Value: Boolean);
begin
if (FCustomColoring <> Value) then
  begin
  FCustomColoring := Value;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetWidthRatio(Value: SmallInt);
begin
if (FWidthRatio <> Value) then
  begin
  FWidthRatio := Value;
  UpdateMaxSize;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetHeightRatio(Value: SmallInt);
begin
if (FHeightRatio <> Value) then
  begin
  FHeightRatio := Value;
  UpdateMaxSize;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetRotationAngle(Value: SmallInt);
begin
if (FRotationAngle <> Value) then
  begin
  FRotationAngle := Value;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetCustomMapping(Value: Boolean);
begin
if (FCustomMapping <> Value) then
  begin
  FCustomMapping := Value;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.CustomMap(var x, y: Single);
var c, u: Single;
begin
if Assigned(FOwner.FCustomMapping) then
  begin
  FOwner.FCustomMapping(FOwner, Self, x, y, c, u);
  x := c;
  y := u;
  end;
end;

procedure TdihavButtonEffect.UpdateMaxSize;
var
  x, y: Integer;
  a: Single;
begin
a := 1;
if (FOwner.ConsiderRotationSize) then a := Sqrt(2);
x := Round(FImgW * FWidthRatio * a / 100) + Abs(FMoveX) * 2;
y := Round(FImgH * FHeightRatio * a / 100) + Abs(FMoveY) * 2;
if (FShadow) then
  begin
  x := x + 2 * Abs(FShadowX);
  y := y + 2 * Abs(FShadowY);
  end;
if ((x <> FMaxSize.cx) or (y <> FMaxSize.cy)) then
  begin
  FMaxSize.cx := x;
  FMaxSize.cy := y;
  FOwner.UpdateAutoSize;
  end;
end;

procedure TdihavButtonEffect.SetColorInversionR(Value: Byte);
begin
if (FColorInversionR <> Value) then
  begin
  FColorInversionR := Value;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetColorInversionG(Value: Byte);
begin
if (FColorInversionG <> Value) then
  begin
  FColorInversionG := Value;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetColorInversionB(Value: Byte);
begin
if (FColorInversionB <> Value) then
  begin
  FColorInversionB := Value;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetAnimationScript(Value: TStrings);
begin
FAnimationScript.Assign(Value);
ResetAnimation;
end;

procedure TdihavButtonEffect.SetAnimatedEffect(Value: Boolean);
begin
if (FAnimatedEffect <> Value) then
  begin
  FAnimatedEffect := Value;
  ResetAnimation;
  end;
end;

procedure TdihavButtonEffect.SetReplayAnimation(Value: Boolean);
begin
if (FReplayAnimation <> Value) then
  begin
  FReplayAnimation := Value;
  ResetAnimation;
  end;
end;

procedure TdihavButtonEffect.ResetAnimation;
begin
Application.ProcessMessages;
FAnimationScriptPos := 0;
ContinueAnimation;
end;

procedure TdihavButtonEffect.ContinueAnimation;
var
  i, j: Integer;
  p, v, s: String;

  procedure NxtFrm;
  begin
  Application.ProcessMessages;
  Inc(FAnimationScriptPos);
  if ((FAnimationScriptPos = FAnimationScript.Count) and FReplayAnimation) then
    begin
    FAnimationScriptPos := 0;
    Application.ProcessMessages;
    end;
  end;

begin
if (not FAnimatedEffect) then Exit;
if (csDesigning in FOwner.ComponentState) then Exit;
if (FOwner.CurrentEffect <> Self) then Exit;
while (FAnimationScriptPos < FAnimationScript.Count) do
  begin
  v := Trim(FAnimationScript.Strings[FAnimationScriptPos]);
  if (v <> '') then
    begin
    if (v = '-') then
      begin
      NxtFrm;
      ApplyChangesNow := True;
      Exit;
      end;
    if (v[1] = '*') then
      begin
      FApplyChangesNow := False;
      Delete(v, 1, 1);
      if (Assigned(FOwner.FCustomAnimationScriptEvent)) then
        FOwner.FCustomAnimationScriptEvent(FOwner, Self, FAnimationScriptPos + 1, v);
      NxtFrm;
      Continue;
      end;
    i := StrToIntDef(v, -1);
    if (i <> - 1) then
      begin
      FApplyChangesNow := False;
      FTimer.Interval := i;
      FTimer.Enabled := True;
      NxtFrm;
      Exit;
      end
    else
      begin
      try
        j := Pos(':=', v);
        if (j = 0) then raise Exception.Create(':= not found.');
        p := Trim(Copy(v, 1, j - 1));
        Delete(v, 1, j + 1);
        v := Trim(v);
        if (v[Length(v)] = ';') then Delete(v, Length(v), 1);
        v := Trim(v);
        if (Lowercase(v) = 'false') then v := '0';
        if (Lowercase(v) = 'true') then v := '1';
        FApplyChangesNow := False;
        SetOrdProp(Self, p, StrToInt(v));
      except
        on e: Exception do
          begin
          s := 'Error in ' + FOwner.Name + '.' + GetName + '.AnimationScript (Line ' + IntToStr(FAnimationScriptPos + 1) + '):'#10#13;
          NxtFrm;
          raise EScriptError.Create(s + e.Message);
          Exit;
          end;
      end;
      end;
    end;
  NxtFrm;
  end;
end;

procedure TdihavButtonEffect.TimerTick(Sender: TObject);
begin
FTimer.Enabled := False;
ContinueAnimation;
end;

procedure TdihavButtonEffect.SetBrightness(Value: SmallInt);
begin
if (FBrightness <> Value) then
  begin
  FBrightness := Value;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetContrast(Value: SmallInt);
begin
if (FContrast <> Value) then
  begin
  FContrast := Value;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetShadow(Value: Boolean);
begin
if (FShadow <> Value) then
  begin
  FShadow := Value;
  UpdateMaxSize;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetShadowColor(Value: TColor);
begin
if (FShadowColor <> Value) then
  begin
  FShadowColor := Value;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetShadowX(Value: Integer);
begin
if (FShadowX <> Value) then
  begin
  FShadowX := Value;
  UpdateMaxSize;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetShadowY(Value: Integer);
begin
if (FShadowY <> Value) then
  begin
  FShadowY := Value;
  UpdateMaxSize;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetShadowAlpha(Value: Byte);
begin
if (FShadowAlpha <> Value) then
  begin
  FShadowAlpha := Value;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetShadowBlurRadius(Value: Byte);
begin
if (FShadowBlurRadius <> Value) then
  begin
  FShadowBlurRadius := Value;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetOverlayImage(Value: Boolean);
begin
if (FOverlayImage <> Value) then
  begin
  FOverlayImage := Value;
  GraphChanged;
  end;
end;

procedure TdihavButtonEffect.AssignTo(Dest: TPersistent);
var d: TdihavButtonEffect;
begin
if ((Dest <> nil) and (Dest is TdihavButtonEffect)) then
  begin
  d := Dest as TdihavButtonEffect;
  d.ApplyChangesNow := False;
  d.Picture := fPicture;
  d.ImageIndex := FImageIndex;
  d.ColorShiftR := FColorShiftR;
  d.ColorShiftG := FColorShiftG;
  d.ColorShiftB := FColorShiftB;
  d.ColorShiftA := FColorShiftA;
  d.ColorPercentageR := FColorRatioR;
  d.ColorPercentageG := FColorRatioG;
  d.ColorPercentageB := FColorRatioB;
  d.ColorPercentageA := FColorRatioA;
  d.ColorShiftH := FColorShiftH;
  d.ColorShiftS := FColorShiftS;
  d.ColorShiftV := FColorShiftV;
  d.ColorPercentageH := FColorRatioH;
  d.ColorPercentageS := FColorRatioS;
  d.ColorPercentageV := FColorRatioV;
  d.InvertColors := FInvertColors;
  d.GrayScale := FGrayScale;
  d.Masked := FMasked;
  d.MaskColor := FMaskColor;
  d.CustomColoring := FCustomColoring;
  d.WidthPercentage := FWidthRatio;
  d.HeightPercentage := FHeightRatio;
  d.RotationAngle := FRotationAngle;
  d.CustomMapping := FCustomMapping;
  d.ColorInversionR := FColorInversionR;
  d.ColorInversionG := FColorInversionG;
  d.ColorInversionB := FColorInversionB;
  d.AnimateFromLastState := FAnimateFromLastState;
  d.AnimationInterval := FAnimationInterval;
  d.AnimationDuration := FFAnimationDuration;
  d.AnimationScript := FAnimationScript;
  d.AnimatedEffect := FAnimatedEffect;
  d.ReplayAnimation := FReplayAnimation;
  d.Brightness := FBrightness;
  d.Contrast := FContrast;
  d.Shadow := FShadow;
  d.ShadowColor := FShadowColor;
  d.ShadowX := FShadowX;
  d.ShadowY := FShadowY;
  d.ShadowAlpha := FShadowAlpha;
  d.ShadowBlurRadius := FShadowBlurRadius;
  d.OverlayImage := FOverlayImage;
  d.MoveX := FMoveX;
  d.MoveY := FMoveY;
  d.ApplyChangesNow := True;
  end;
end;

function TdihavButtonEffect.GetName: String;
begin
case FType of
  etNormal: Result := 'EffectNormal';
  etActive: Result := 'EffectActive';
  etOver: Result := 'EffectOver';
  etDown: Result := 'EffectDown';
  etFocused: Result := 'EffectFocused';
  etDisabled: Result := 'EffectDisabled';
end;
end;

procedure TdihavButtonEffect.SetMoveX(Value: Integer);
begin
if (FMoveX <> Value) then
  begin
  FMoveX := Value;
  UpdateMaxSize;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetMoveY(Value: Integer);
begin
if (FMoveY <> Value) then
  begin
  FMoveY := Value;
  UpdateMaxSize;
  PropertyChanged;
  end;
end;

procedure TdihavButtonEffect.SetApplyChangesNow(Value: Boolean);
begin
if (FApplyChangesNow <> Value) then
  begin
  FApplyChangesNow := Value;
  if (Value) then PropertyChanged;  
  end;
end;

// TdihavButtonActionLink

procedure TdihavButtonActionLink.AssignClient(AClient: TObject);
begin
inherited AssignClient(AClient);
FClient := AClient as TdihavButton;
end;

constructor TdihavButtonActionLink.Create(AClient: TObject);
begin
inherited Create(AClient);
end;

function TdihavButtonActionLink.IsCheckedLinked: Boolean;
begin
Result := ((inherited IsCheckedLinked) and (FClient.GroupIndex <> 0) and (FClient.AllowAllUp) and (FClient.Down = (Action as TCustomAction).Checked));
end;

function TdihavButtonActionLink.IsGroupIndexLinked: Boolean;
begin
Result := ((inherited IsGroupIndexLinked) and (FClient.GroupIndex = (Action as TCustomAction).GroupIndex));
end;

function TdihavButtonActionLink.IsImageIndexLinked: Boolean;
begin
Result := ((inherited IsImageIndexLinked) and (FClient.ImageIndex = (Action as TCustomAction).ImageIndex));
end;

procedure TdihavButtonActionLink.SetChecked(Value: Boolean);
begin
if IsCheckedLinked then FClient.Down := Value;
end;

procedure TdihavButtonActionLink.SetGroupIndex(Value: Integer);
begin
if IsGroupIndexLinked then FClient.GroupIndex := Value;
end;

procedure TdihavButtonActionLink.SetImageIndex(Value: Integer);
begin
if IsImageIndexLinked then FClient.ImageIndex := Value;
end;

// TdihavButton

function TdihavButton.GetHomePage: String;
begin
Result := 'http://www.dihav.com/';
end;

procedure TdihavButton.SetHomePage(Value: String);
begin
//No Code
end;

procedure TdihavButton.Paint;
var
  b: TBitmap;
  e: TdihavButtonEffect;
begin
inherited Paint;
if (Assigned(Parent) and ((Visible) or (csDesigning in ComponentState))) then
  begin
  b := TBitmap.Create;
  b.Width := Width;
  b.Height := Height;
  b.Canvas.Brush.Color := Color;
  b.Canvas.FloodFill(0, 0, clWhite, fsSurface);
  if (csDesigning in ComponentState) then
    begin
    b.Canvas.Pen.Style := psDash;
    b.Canvas.Brush.Style := bsClear;
    b.Canvas.Rectangle(0, 0, Width, Height);
    end
  else if (FCustomBackground and Assigned(FCustomBackgroundEvent)) then
    FCustomBackgroundEvent(Self, b.Canvas, b.Canvas.ClipRect);
  b.PixelFormat := pf24bit;
  if (bsDisabled in FButtonState) then
    e := FEffectDisabled
  else if ((bsDown in FButtonState) and (not CheckButtonState)) then
    e := FEffectDown
  else if (bsOver in FButtonState) then
    e := FEffectOver
  else if (bsFocused in FButtonState) then
    e := FEffectFocused
  else if (bsActive in FButtonState) then
    e := FEffectActive
  else if ((bsDown in FButtonState) and (CheckButtonState)) then
    e := FEffectDown
  else
    e := FEffectNormal;
  if (e <> FCurrentEffect) then
    begin
    FCurrentEffect := e;
    FCurrentEffect.ResetAnimation;
    FCurrentEffect.Repaint;
    end
  else
    begin
    FCurrentEffect.Draw(b);
    Canvas.Draw(0, 0, b);
    end;
  b.Free;
  end;
end;

constructor TdihavButton.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
ControlStyle := ControlStyle + [csSetCaption, csOpaque];
Brush.Style := bsClear;
Width := 48;
Height := 48;
AutoSize := True;
TabStop := True;
FButtonState := [];
FDefault := False;
FCancel := False;
FActive := False;
FDown := False;
FGroupIndex := 0;
FAllowAllUp := False;
FModalResult := mrNone;
FClicksDisabled := False;
FImageList := nil;
FImageChangeLink := TChangeLink.Create;
FImageChangeLink.OnChange := ImgChange;
FImageIndex := -1;
FPicture := TPicture.Create;
FPicture.OnChange := ImgChange;
FEffectNormal := TdihavButtonEffect.Create(Self, etNormal);
FEffectOver := TdihavButtonEffect.Create(Self, etOver);
FEffectDown := TdihavButtonEffect.Create(Self, etDown);
FEffectFocused := TdihavButtonEffect.Create(Self, etFocused);
FEffectDisabled := TdihavButtonEffect.Create(Self, etDisabled);
FEffectActive := TdihavButtonEffect.Create(Self, etActive);
FCurrentEffect := nil;
FCustomMapping := nil;
FCustomColoring := nil;
FConsiderRotationSize := True;
FAnimationPos := 0;
FTimer := TTImer.Create(Self);
FTimer.Enabled := False;
FTimer.OnTimer := TimerTick;
FImageShape := False;
FRGN := 0;
FCaptionAlignment := taCenter;
FCaptionLayout := tlCenter;
FCustomBackground := False;
FCustomBackgroundEvent := nil;
FCustomAnimationScriptEvent := nil;
FButtonStateChangedEvent := nil;
FCheckButtonState := False;
FAutoCheckButtonState := True;
FNormalImgR := TBitmap.Create;
FNormalImgB := TBitmap.Create;
FDownImgR := TBitmap.Create;
FDownImgB := TBitmap.Create;
FNormalImgR.Canvas.Pen.Color := clRed;
FNormalImgR.Canvas.Brush.Color := clRed;
FNormalImgB.Canvas.Pen.Color := clBlue;
FNormalImgB.Canvas.Brush.Color := clBlue;
FDownImgR.Canvas.Pen.Color := clRed;
FDownImgR.Canvas.Brush.Color := clRed;
FDownImgB.Canvas.Pen.Color := clBlue;
FDownImgB.Canvas.Brush.Color := clBlue;
FNormalImgR.PixelFormat := pf24bit;
FNormalImgB.PixelFormat := pf24bit;
FDownImgR.PixelFormat := pf24bit;
FDownImgB.PixelFormat := pf24bit;
FAssigningEffects := False;
end;

destructor TdihavButton.Destroy;
begin
FImageChangeLink.Free;               
FPicture.Free;
FEffectNormal.Free;
FEffectOver.Free;
FEffectDown.Free;
FEffectFocused.Free;
FEffectDisabled.Free;
FEffectActive.Free;
FTimer.Free;
if (FRGN <> 0) then DeleteObject(FRGN);
FNormalImgR.Free;
FNormalImgB.Free;
FDownImgR.Free;
FDownImgB.Free;
inherited Destroy;
end;

procedure TdihavButton.Click;
var f: TCustomForm;
begin
if (not Enabled) then Exit;
f := GetParentForm(Self);
if (f <> nil) then f.ModalResult := ModalResult;
if (FGroupIndex <> 0) then
  SetDown(not(FAllowAllUp and FDown));
inherited Click;
end;

procedure TdihavButton.DblClick;
begin
if Enabled then inherited DblClick;
end;

procedure TdihavButton.SetDefault(Value: Boolean);
var f: TCustomForm;
begin
FDefault := Value;
if HandleAllocated then
  begin
  f := GetParentForm(Self);
  if (f <> nil) then f.Perform(CM_FOCUSCHANGED, 0, Longint(f.ActiveControl));
  end;
end;

procedure TdihavButton.CreateWnd;
begin
inherited CreateWnd;
FActive := FDefault;
end;

procedure TdihavButton.CNCommand(var Message: TWMCommand);
begin
if (Message.NotifyCode = BN_CLICKED) then Click;
inherited;
end;

procedure TdihavButton.CMDialogChar(var Message: TCMDialogChar);
begin
with Message do
  if (IsAccel(CharCode, Caption) and Enabled and Visible and (Parent <> nil) and Parent.Showing) then
    begin
    Click;
    Result := 1;
    end
  else
    inherited;
end;

procedure TdihavButton.CMDialogKey(var Message: TCMDialogKey);
begin
with Message do
  if ((((CharCode = VK_RETURN) and FActive) or
       ((CharCode = VK_ESCAPE) and FCancel)) and
      (KeyDataToShiftState(Message.KeyData) = []) and CanFocus) then
    begin
    Click;
    Result := 1;
    end
  else
    inherited;
end;

procedure TdihavButton.CMFocusChanged(var Message: TCMFocusChanged);
begin
with Message do
  if ((Sender is TButton) or (Sender is TdihavButton)) then
    FActive := (Sender = Self)
  else
    FActive := FDefault;
if FActive then
  ButtonState := ButtonState + [bsActive]
else
  ButtonState := ButtonState - [bsActive];
inherited;
end;

procedure TdihavButton.SetButtonState(Value: TdihavButtonState);
var l: TdihavButtonState;
begin
if (Value <> FButtonState) then
  begin
  l := FButtonState;
  FButtonState := Value;
  if (FDown) then FButtonState := FButtonState + [bsDown];
  if (Assigned(FButtonStateChangedEvent)) then FButtonStateChangedEvent(Self, l);
  Paint;
  end;
end;

procedure TdihavButton.WndProc(var Message: TMessage);
begin
case Message.Msg of
  WM_LBUTTONDOWN, WM_LBUTTONDBLCLK:
    if not (csDesigning in ComponentState) and not Focused then
      begin
      FClicksDisabled := True;
      Windows.SetFocus(Handle);
      FClicksDisabled := False;
      if not Focused then Exit;
      end;
  CN_COMMAND:
    if FClicksDisabled then Exit;
  end;
inherited WndProc(Message);
end;

procedure TdihavButton.SetDown(Value: Boolean);
begin
if (FGroupIndex = 0) then Value := False;
if (Value <> FDown) then
  begin
  if (FDown and (not FAllowAllUp)) then Exit;
  FDown := Value;
  if (Action is TCustomAction) then TCustomAction(Action).Checked := FDown;
  UpdateStateBackImagesNoImgChange;
  if Value then
    begin
    ButtonState := ButtonState + [bsDown];
    UpdateExclusive;
    end
  else
    ButtonState := ButtonState - [bsDown];
  end;
end;

function TdihavButton.GetActionLinkClass: TControlActionLinkClass;
begin
Result := TdihavButtonActionLink;
end;

procedure TdihavButton.UpdateExclusive;
var Msg: TMessage;
begin
if ((FGroupIndex <> 0) and (Parent <> nil)) then
  begin
  Msg.Msg := CM_BUTTONPRESSED;
  Msg.WParam := FGroupIndex;
  Msg.LParam := Longint(Self);
  Msg.Result := 0;
  Parent.Broadcast(Msg);
  end;
end;

procedure TdihavButton.SetGroupIndex(Value: Integer);
begin
if (FGroupIndex <> Value) then
  begin
  FGroupIndex := Value;
  if (FGroupIndex = 0) then
    begin
    FDown := False;
    if (Action is TCustomAction) then TCustomAction(Action).Checked := False;
    UpdateStateBackImagesNoImgChange;
    ButtonState := ButtonState - [bsDown];
    end;
  if (FAutoCheckButtonState) then
    begin
    CheckButtonState := (FGroupIndex <> 0);
    FAutoCheckButtonState := True;
    end;
  UpdateExclusive;
  end;
end;

procedure TdihavButton.SetAllowAllUp(Value: Boolean);
begin
if (FAllowAllUp <> Value) then
  begin
  FAllowAllUp := Value;
  UpdateExclusive;
  end;
end;

procedure TdihavButton.CMEnabledChanged(var Message: TMessage);
begin
inherited;
if Enabled then
  ButtonState := ButtonState - [bsDisabled]
else
  ButtonState := ButtonState + [bsDisabled] - [bsFocused, bsActive];
end;

procedure TdihavButton.CMButtonPressed(var Message: TMessage);
var s: TdihavButton;
begin
if (Message.WParam = FGroupIndex) then
  begin
  if not (TComponent(Message.LParam) is TdihavButton) then Exit;
  s := TdihavButton(Message.LParam);
  if (s <> Self) then
    begin
    if (s.Down and FDown) then
      begin
      FDown := False;
      if (Action is TCustomAction) then TCustomAction(Action).Checked := False;
      UpdateStateBackImagesNoImgChange;
      ButtonState := ButtonState - [bsDown];
      end;
    FAllowAllUp := s.AllowAllUp;
    end;
  end;
end;

procedure TdihavButton.CMSysColorChange(var Message: TMessage);
begin
inherited;
Invalidate;
end;

procedure TdihavButton.CMMouseEnter(var Message: TMessage);
begin
ButtonState := ButtonState + [bsOver];
inherited;
end;

procedure TdihavButton.CMMouseLeave(var Message: TMessage);
begin
ButtonState := ButtonState - [bsOver];
inherited;
end;

procedure TdihavButton.ActionChange(Sender: TObject; CheckDefaults: Boolean);
begin
inherited ActionChange(Sender, CheckDefaults);
if (Sender is TCustomAction) then
  with TCustomAction(Sender) do
    begin
    if (CheckDefaults or (Self.ImageIndex <> ImageIndex)) then
      Self.ImageIndex := ImageIndex;
    if (CheckDefaults or (Self.GroupIndex <> GroupIndex)) then
      Self.GroupIndex := GroupIndex;
    FAllowAllUp := True;
    SetDown(Checked);
    GraphsChanged;
    end;
end;

procedure TdihavButton.Loaded;
var b: TdihavButtonState;
begin
inherited Loaded;
GraphsChanged;
b := [];
if (not Enabled) then b := [bsDisabled];
if (FDefault) then b := b + [bsActive];
if (FDown) then b := b + [bsDown];
if (Focused) then b := b + [bsFocused];
if (bsDisabled in b) then
  FCurrentEffect := FEffectDisabled
else if ((bsDown in b) and (not CheckButtonState)) then
  FCurrentEffect := FEffectDown
else if (bsOver in b) then
  FCurrentEffect := FEffectOver
else if (bsFocused in b) then
  FCurrentEffect := FEffectFocused
else if (bsActive in b) then
  FCurrentEffect := FEffectActive
else if ((bsDown in b) and (CheckButtonState)) then
  FCurrentEffect := FEffectDown
else
  FCurrentEffect := FEffectNormal;
AssignEffectVars(FCurrentEffect);
FTimer.Enabled := False;
ButtonState := ButtonState + b;
FCurrentEffect.Repaint;
end;

procedure TdihavButton.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
if not Enabled then Exit;
if ((Button = mbLeft) and Enabled) then
  ButtonState := ButtonState + [bsDown];
inherited MouseDown(Button, Shift, X, Y);
end;

procedure TdihavButton.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
if not Enabled then Exit;
if ((Button = mbLeft) and Enabled) then
  if ((not FDown) and (not CheckButtonState)) then
    ButtonState := ButtonState - [bsDown];
inherited MouseUp(Button, Shift, X, Y);
end;

procedure TdihavButton.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
if Enabled then inherited MouseMove(Shift, X, Y);
end;

procedure TdihavButton.CMGotFocus(var Message: TMessage);
begin
if CanFocus then ButtonState := ButtonState + [bsFocused];
inherited;
end;

procedure TdihavButton.CMLostFocus(var Message: TMessage);
begin
ButtonState := ButtonState - [bsFocused];
if (not FDown) then ButtonState := ButtonState - [bsDown];
inherited;
end;

procedure TdihavButton.CMColorChanged(var Message: TMessage);
begin
inherited;
Invalidate;
end;

procedure TdihavButton.WMKeyUp(var Message:TWMKeyUp);
begin
if (Message.CharCode = VK_SPACE) then
  begin
  Click;
  if ((not FDown) and (not CheckButtonState)) then
    ButtonState := ButtonState - [bsDown];
  end;
inherited;
end;

procedure TdihavButton.WMKeyDown(var Message:TWMKeyDown);
begin
if (Message.CharCode = VK_SPACE) then
  ButtonState := ButtonState + [bsDown];
inherited;
end;

procedure TdihavButton.Notification(AComponent: TComponent; Operation: TOperation);
begin
if (csDestroying in ComponentState) then
  begin
  inherited Notification(AComponent,Operation);
  Exit;
  end;
if ((Operation = opRemove) and (AComponent = Action)) then
  begin
  Action := nil;
  GraphsChanged;
  end;
if ((Operation = opRemove) and (AComponent = FImageList)) then
  begin
  FImageList := nil;
  GraphsChanged;
  end;
inherited Notification(AComponent,Operation);
end;

procedure TdihavButton.SetImageList(Value: TCustomImageList);
begin
if (Value <> FImageList) then
  begin
  if Assigned(FImageList) then FImageList.UnRegisterChanges(FImageChangeLink);
  FImageList := Value;
  if (FImageList <> nil) then
    begin
    FImageList.RegisterChanges(FImageChangeLink);
    FImageList.FreeNotification(Self);
    end;
  GraphsChanged;
  end;
end;

procedure TdihavButton.ImgChange(Sender: TObject);
begin
GraphsChanged;
end;

procedure TdihavButton.SetImageIndex(Value: TImageIndex);
begin
if (FImageIndex <> Value) then
  begin
  FImageIndex := Value;
  GraphsChanged;
  end;
end;

procedure TdihavButton.GraphsChanged;
begin
FEffectNormal.GraphChanged;
FEffectDown.GraphChanged;
FEffectOver.GraphChanged;
FEffectFocused.GraphChanged;
FEffectDisabled.GraphChanged;
FEffectActive.GraphChanged;
Paint;
end;

function TdihavButton.CanAutoSize(var NewWidth, NewHeight: Integer): Boolean;
var x, y: Integer;
begin
Result := True;
x := FEffectNormal.FMaxSize.cx;
if (x < FEffectOver.FMaxSize.cx) then x := FEffectOver.FMaxSize.cx;
if (x < FEffectDown.FMaxSize.cx) then x := FEffectDown.FMaxSize.cx;
if (x < FEffectFocused.FMaxSize.cx) then x := FEffectFocused.FMaxSize.cx;
if (x < FEffectDisabled.FMaxSize.cx) then x := FEffectDisabled.FMaxSize.cx;
if (x < FEffectActive.FMaxSize.cx) then x := FEffectActive.FMaxSize.cx;
y := FEffectNormal.FMaxSize.cy;
if (y < FEffectOver.FMaxSize.cy) then y := FEffectOver.FMaxSize.cy;
if (y < FEffectDown.FMaxSize.cy) then y := FEffectDown.FMaxSize.cy;
if (y < FEffectFocused.FMaxSize.cy) then y := FEffectFocused.FMaxSize.cy;
if (y < FEffectDisabled.FMaxSize.cy) then y := FEffectDisabled.FMaxSize.cy;
if (y < FEffectActive.FMaxSize.cy) then y := FEffectActive.FMaxSize.cy;
if ((x = 0) and (y = 0)) then Exit;
if FConsiderRotationSize then
  begin
  if (x > y) then y := x;
  if (y > x) then x := y;
  end;
if (Align in [alNone, alLeft, alRight]) then
  NewWidth := x;
if (Align in [alNone, alTop, alBottom]) then
  NewHeight := y;
end;

procedure TdihavButton.SetPicture(Value: TPicture);
begin
FPicture.Assign(Value);
FPicture.OnChange := ImgChange;
GraphsChanged;
end;

procedure TdihavButton.UpdateAutoSize;
begin
if AutoSize then AdjustSize;
end;

procedure TdihavButton.SetConsiderRotationSize(Value: Boolean);
begin
if (FConsiderRotationSize <> Value) then
  begin
  FConsiderRotationSize := Value;
  FEffectNormal.UpdateMaxSize;
  FEffectOver.UpdateMaxSize;
  FEffectDown.UpdateMaxSize;
  FEffectFocused.UpdateMaxSize;
  FEffectDisabled.UpdateMaxSize;
  FEffectActive.UpdateMaxSize;
  end;
end;

procedure TdihavButton.SaveEffectVars;
begin
_FColorShiftR := FColorShiftR;
_FColorShiftG := FColorShiftG;
_FColorShiftB := FColorShiftB;
_FColorShiftA := FColorShiftA;
_FColorRatioR := FColorRatioR;
_FColorRatioG := FColorRatioG;
_FColorRatioB := FColorRatioB;
_FColorRatioA := FColorRatioA;
_FColorShiftH := FColorShiftH;
_FColorShiftS := FColorShiftS;
_FColorShiftV := FColorShiftV;
_FColorRatioH := FColorRatioH;
_FColorRatioS := FColorRatioS;
_FColorRatioV := FColorRatioV;
_FInvertColors := FInvertColors;
_FGrayScale := FGrayScale;
_FMasked := FMasked;
_FMaskColor := FMaskColor;
_FWidthRatio := FWidthRatio;
_FHeightRatio := FHeightRatio;
_FRotationAngle := FRotationAngle;
_FColorInversionR := FColorInversionR;
_FColorInversionG := FColorInversionG;
_FColorInversionB := FColorInversionB;
_FBrightness := FBrightness;
_FContrast := FContrast;
_FShadow := FShadow;
_FShadowColor := FShadowColor;
_FShadowX := FShadowX;
_FShadowY := FShadowY;
_FShadowAlpha := FShadowAlpha;
_FShadowBlurRadius := FShadowBlurRadius;
_FMoveX := FMoveX;
_FMoveY := FMoveY;
end;

procedure TdihavButton.RestoreEffectVars;
begin
FColorShiftR := _FColorShiftR;
FColorShiftG := _FColorShiftG;
FColorShiftB := _FColorShiftB;
FColorShiftA := _FColorShiftA;
FColorRatioR := _FColorRatioR;
FColorRatioG := _FColorRatioG;
FColorRatioB := _FColorRatioB;
FColorRatioA := _FColorRatioA;
FColorShiftH := _FColorShiftH;
FColorShiftS := _FColorShiftS;
FColorShiftV := _FColorShiftV;
FColorRatioH := _FColorRatioH;
FColorRatioS := _FColorRatioS;
FColorRatioV := _FColorRatioV;
FInvertColors := _FInvertColors;
FGrayScale := _FGrayScale;
FMasked := _FMasked;
FMaskColor := _FMaskColor;
FWidthRatio := _FWidthRatio;
FHeightRatio := _FHeightRatio;
FRotationAngle := _FRotationAngle;
FColorInversionR := _FColorInversionR;
FColorInversionG := _FColorInversionG;
FColorInversionB := _FColorInversionB;
FBrightness := _FBrightness;
FContrast := _FContrast;
FShadow := _FShadow;
FShadowColor := _FShadowColor;
FShadowX := _FShadowX;
FShadowY := _FShadowY;
FShadowAlpha := _FShadowAlpha;
FShadowBlurRadius := _FShadowBlurRadius;
FMoveX := _FMoveX;
FMoveY := _FMoveY;
end;

procedure TdihavButton.AssignEffectVars(AEffect: TdihavButtonEffect);
begin
FColorShiftR := AEffect.ColorShiftR;
FColorShiftG := AEffect.ColorShiftG;
FColorShiftB := AEffect.ColorShiftB;
FColorShiftA := AEffect.ColorShiftA;
FColorRatioR := AEffect.ColorPercentageR;
FColorRatioG := AEffect.ColorPercentageG;
FColorRatioB := AEffect.ColorPercentageB;
FColorRatioA := AEffect.ColorPercentageA;
FColorShiftH := AEffect.ColorShiftH;
FColorShiftS := AEffect.ColorShiftS;
FColorShiftV := AEffect.ColorShiftV;
FColorRatioH := AEffect.ColorPercentageH;
FColorRatioS := AEffect.ColorPercentageS;
FColorRatioV := AEffect.ColorPercentageV;
FInvertColors := AEffect.InvertColors;
FGrayScale := AEffect.GrayScale;
FMasked := AEffect.Masked;
FMaskColor := AEffect.MaskColor;
FWidthRatio := AEffect.WidthPercentage;
FHeightRatio := AEffect.HeightPercentage;
FRotationAngle := AEffect.RotationAngle;
FColorInversionR := AEffect.ColorInversionR;
FColorInversionG := AEffect.ColorInversionG;
FColorInversionB := AEffect.ColorInversionB;
FBrightness := AEffect.Brightness;
FContrast := AEffect.Contrast;
FShadow := AEffect.Shadow;
FShadowColor := AEffect.ShadowColor;
FShadowX := AEffect.ShadowX;
FShadowY := AEffect.ShadowY;
FShadowAlpha := AEffect.ShadowAlpha;
if (not AEffect.Shadow) then
  begin
  FShadowX := 0;
  FShadowY := 0;
  FShadowAlpha := 0;
  end;
FShadowBlurRadius := AEffect.ShadowBlurRadius;
FMoveX := AEffect.MoveX;
FMoveY := AEffect.MoveY;
end;

procedure TdihavButton.CalcEffectDelta(AEffect: TdihavButtonEffect);
var c1, c2: TColor;
begin
FColorShiftRD := AEffect.ColorShiftR - FColorShiftR;
FColorShiftGD := AEffect.ColorShiftG - FColorShiftG;
FColorShiftBD := AEffect.ColorShiftB - FColorShiftB;
FColorShiftAD := AEffect.ColorShiftA - FColorShiftA;
FColorRatioRD := AEffect.ColorPercentageR - FColorRatioR;
FColorRatioGD := AEffect.ColorPercentageG - FColorRatioG;
FColorRatioBD := AEffect.ColorPercentageB - FColorRatioB;
FColorRatioAD := AEffect.ColorPercentageA - FColorRatioA;
FColorShiftHD := AEffect.ColorShiftH - FColorShiftH;
FColorShiftSD := AEffect.ColorShiftS - FColorShiftS;
FColorShiftVD := AEffect.ColorShiftV - FColorShiftV;
FColorRatioHD := AEffect.ColorPercentageH - FColorRatioH;
FColorRatioSD := AEffect.ColorPercentageS - FColorRatioS;
FColorRatioVD := AEffect.ColorPercentageV - FColorRatioV;
FInvertColorsD := (AEffect.InvertColors <> FInvertColors);
FGrayScaleD := (AEffect.GrayScale <> FGrayScale);
FMaskedD := (AEffect.Masked <> FMasked);
c1 := ColorToRGB(AEffect.MaskColor);
c2 := ColorToRGB(FMaskColor);
FMaskColorRD := GetRValue(c1) - GetRValue(c2);
FMaskColorGD := GetGValue(c1) - GetGValue(c2);
FMaskColorBD := GetBValue(c1) - GetBValue(c2);
FWidthRatioD := AEffect.WidthPercentage - FWidthRatio;
FHeightRatioD := AEffect.HeightPercentage - FHeightRatio;
FRotationAngleD := AEffect.RotationAngle - FRotationAngle;
FColorInversionRD := AEffect.ColorInversionR - FColorInversionR;
FColorInversionGD := AEffect.ColorInversionG - FColorInversionG;
FColorInversionBD := AEffect.ColorInversionB - FColorInversionB;
FBrightnessD := AEffect.Brightness - FBrightness;
FContrastD := AEffect.Contrast - FContrast;
c1 := ColorToRGB(AEffect.ShadowColor);
c2 := ColorToRGB(FShadowColor);
FShadowColorRD := GetRValue(c1) - GetRValue(c2);
FShadowColorGD := GetGValue(c1) - GetGValue(c2);
FShadowColorBD := GetBValue(c1) - GetBValue(c2);
FShadowXD := AEffect.ShadowX - FShadowX;
FShadowYD := AEffect.ShadowY - FShadowY;
FShadowAlphaD := AEffect.ShadowAlpha - FShadowAlpha;
if (not AEffect.Shadow) then
  begin
  FShadowXD := -FShadowX;
  FShadowYD := -FShadowY;
  FShadowAlphaD := -FShadowAlpha;
  end;
FShadowBlurRadiusD := AEffect.ShadowBlurRadius - FShadowBlurRadius;
FMoveXD := AEffect.MoveX - FMoveX;
FMoveYD := AEffect.MoveY - FMoveY;

FColorShiftR0 := FColorShiftR;
FColorShiftG0 := FColorShiftG;
FColorShiftB0 := FColorShiftB;
FColorShiftA0 := FColorShiftA;
FColorRatioR0 := FColorRatioR;
FColorRatioG0 := FColorRatioG;
FColorRatioB0 := FColorRatioB;
FColorRatioA0 := FColorRatioA;
FColorShiftH0 := FColorShiftH;
FColorShiftS0 := FColorShiftS;
FColorShiftV0 := FColorShiftV;
FColorRatioH0 := FColorRatioH;
FColorRatioS0 := FColorRatioS;
FColorRatioV0 := FColorRatioV;
FMaskColorR0 := GetRValue(c2);
FMaskColorG0 := GetGValue(c2);
FMaskColorB0 := GetBValue(c2);
FWidthRatio0 := FWidthRatio;
FHeightRatio0 := FHeightRatio;
FRotationAngle0 := FRotationAngle;
FColorInversionR0 := FColorInversionR;
FColorInversionG0 := FColorInversionG;
FColorInversionB0 := FColorInversionB;
FBrightness0 := FBrightness;
FContrast0 := FContrast;
FShadowColorR0 := GetRValue(c2);
FShadowColorG0 := GetGValue(c2);
FShadowColorB0 := GetBValue(c2);
FShadowX0 := FShadowX;
FShadowY0 := FShadowY;
FShadowAlpha0 := FShadowAlpha;
FShadowBlurRadius0 := FShadowBlurRadius;
FMoveX0 := FMoveX;
FMoveY0 := FMoveY;
end;

procedure TdihavButton.TimerTick(Sender: TObject);
var
  a: Real;
  r, g, b: Byte;
begin
FTimer.Enabled := False;
Application.ProcessMessages;
FTimer.Tag := FTimer.Tag + Integer(GetTickCount - FTickHolder);
FTickHolder := GetTickCount;
if (FTimer.Tag >= Integer(CurrentEffect.AnimationDuration)) then
  begin
  AssignEffectVars(CurrentEffect);
  FAnimationPos := 1;
  Repaint;
  FAnimationPos := 0;
  CurrentEffect.ContinueAnimation;
  end
else
  begin
  a := FTimer.Tag  / CurrentEffect.AnimationDuration;
  FColorShiftR := Round(a * FColorShiftRD + FColorShiftR0);
  FColorShiftG := Round(a * FColorShiftGD + FColorShiftG0);
  FColorShiftB := Round(a * FColorShiftBD + FColorShiftB0);
  FColorShiftA := Round(a * FColorShiftAD + FColorShiftA0);
  FColorRatioR := Round(a * FColorRatioRD + FColorRatioR0);
  FColorRatioG := Round(a * FColorRatioGD + FColorRatioG0);
  FColorRatioB := Round(a * FColorRatioBD + FColorRatioB0);
  FColorRatioA := Round(a * FColorRatioAD + FColorRatioA0);
  FColorShiftH := Round(a * FColorShiftHD + FColorShiftH0);
  FColorShiftS := Round(a * FColorShiftSD + FColorShiftS0);
  FColorShiftV := Round(a * FColorShiftVD + FColorShiftV0);
  FColorRatioH := Round(a * FColorRatioHD + FColorRatioH0);
  FColorRatioS := Round(a * FColorRatioSD + FColorRatioS0);
  FColorRatioV := Round(a * FColorRatioVD + FColorRatioV0);
  r := ToByte(a * FMaskColorRD + FMaskColorR0);
  g := ToByte(a * FMaskColorGD + FMaskColorG0);
  b := ToByte(a * FMaskColorBD + FMaskColorB0);
  FMaskColor := RGB(r, g, b);
  FWidthRatio := Round(a * FWidthRatioD + FWidthRatio0);
  FHeightRatio := Round(a * FHeightRatioD + FHeightRatio0);
  FRotationAngle := Round(a * FRotationAngleD + FRotationAngle0);
  FColorInversionR := ToByte(a * FColorInversionRD + FColorInversionR0);
  FColorInversionG := ToByte(a * FColorInversionGD + FColorInversionG0);
  FColorInversionB := ToByte(a * FColorInversionBD + FColorInversionB0);
  FBrightness := Round(a * FBrightnessD + FBrightness0);
  FContrast := Round(a * FContrastD + FContrast0);
  r := ToByte(a * FShadowColorRD + FShadowColorR0);
  g := ToByte(a * FShadowColorGD + FShadowColorG0);
  b := ToByte(a * FShadowColorBD + FShadowColorB0);
  FShadowColor := RGB(r, g, b);
  FShadowX := Round(a * FShadowXD + FShadowX0);
  FShadowY := Round(a * FShadowYD + FShadowY0);
  FShadowAlpha := ToByte(a * FShadowAlphaD + FShadowAlpha0);
  FShadowBlurRadius := ToByte(a * FShadowBlurRadiusD + FShadowBlurRadius0);
  FMoveX := Round(a * FMoveXD + FMoveX0);
  FMoveY := Round(a * FMoveYD + FMoveY0);
  FAnimationPos := a;
  Repaint;
  FTimer.Enabled := True;
  end;
end;

procedure TdihavButton.SetImageShape(Value: Boolean);
begin
if (FImageShape <> Value) then
  begin
  FImageShape := Value;
  if not (FImageShape or (csDesigning in ComponentState)) then
    begin
    DeleteObject(FRGN);
    FRGN := 0;
    SetWindowRGN(Handle, 0, True);
    end;
  end;
end;

procedure TdihavButton.CMFontChanged(var Message: TMessage);
begin
inherited;
GraphsChanged;
end;

procedure TdihavButton.CMTextChanged(var Message: TMessage);
begin
inherited;
GraphsChanged;
end;

procedure TdihavButton.CMBiDiModeChanged(var Message: TMessage);
begin
inherited;
GraphsChanged;
end;

procedure TdihavButton.SetCaptionAlignment(Value: TAlignment);
begin
if (FCaptionAlignment <> Value) then
  begin
  FCaptionAlignment := Value;
  GraphsChanged;
  end;
end;

procedure TdihavButton.SetCaptionLayout(Value: TTextLayout);
begin
if (FCaptionLayout <> Value) then
  begin
  FCaptionLayout := Value;
  GraphsChanged;
  end;
end;

procedure TdihavButton.SetCustomBackground(Value: Boolean);
begin
if (FCustomBackground <> Value) then
  begin
  FCustomBackground := Value;
  Repaint;
  end;
end;

procedure TdihavButton.AssignEffectsFromButton(Source: TdihavButton);
begin
FAssigningEffects := True;
EffectNormal.Assign(Source.EffectNormal);
EffectDown.Assign(Source.EffectDown);
EffectOver.Assign(Source.EffectOver);
EffectFocused.Assign(Source.EffectFocused);
EffectDisabled.Assign(Source.EffectDisabled);
EffectActive.Assign(Source.EffectActive);
FAssigningEffects := False;
FCurrentEffect.Repaint;
end;

procedure TdihavButton.WMSetFocus(var Message:TWMSetFocus);
begin
if CanFocus then ButtonState := ButtonState + [bsFocused];
inherited;
end;

procedure TdihavButton.WMKillFocus(var Message:TWMKillFocus);
begin
ButtonState := ButtonState - [bsFocused];
if (not FDown) then ButtonState := ButtonState - [bsDown];
inherited;
end;

procedure TdihavButton.SetCheckButtonState(Value: Boolean);
begin
if (FCheckButtonState <> Value) then
  begin
  FCheckButtonState := Value;
  FAutoCheckButtonState := False;
  Repaint;
  end;
end;

function TdihavButton.CheckButtonStateStored: Boolean;
begin
Result := not FAutoCheckButtonState;
end;

procedure TdihavButton.SetAutoCheckButtonState(Value: Boolean);
begin
if (FAutoCheckButtonState <> Value) then
  begin
  if (Value) then CheckButtonState := (FGroupIndex <> 0);
  FAutoCheckButtonState := Value;
  end;
end;

procedure TdihavButton.UpdateStateBackImagesNoImgChange;
begin
if (not CheckButtonState) then Exit;
FEffectOver.GraphChanged;
FEffectFocused.GraphChanged;
FEffectDisabled.GraphChanged;
FEffectActive.GraphChanged;
end;

end.
